@@ -2,17 +2,21 @@
  * Copyright (c) 2007, to-do. All rights reserved.
  */
 #include "util.h"
+#include "var.h"
 #include "_sym.h"
+#include "_ext.h"
 #include "_op.h"
 
 /************************************************************
  *
  ************************************************************/
+ /* LEX FLAG */
 #define LF_MACRO      0x01
 #define LF_ECHO       0x02
 #define LF_RIGHT      0x10
 #define LF_LEFT       0x20
 
+/* VALUE FLAG */
 #define VF_SYMBOL     0x10
 #define VF_REFER      0x20
 #define VF_UNRESOLV   0xF0
@@ -21,6 +25,7 @@
 #define VF_CONST      0x04
 #define VF_RESOLV     0x0F
 
+/* ALLOW FLAG */
 #define AF_BREAK      0x01
 #define AF_CONTINUE   0x02
 #define AF_RETURN     0x04
@@ -82,6 +87,9 @@ typedef struct {
 	int vf; /* value flag */
 	int nc; /* cols */
 	int nr; /* rows */
+	FILE* out;
+	FILE* err;
+	FILE* in;
 } compile_t;
 
 /************************************************************
@@ -177,7 +185,7 @@ static void put_string(compile_t* cp, char *s, int n)
 	n = 0;
 	while ((s < e) && (*s != q)) {
 		if ((c = *s++) == '\\') {
-			c = unmetac(&s, q);
+			c = meta_to_c(&s, q);
 		}
 		d[n++] = c;
 	}
@@ -257,7 +265,7 @@ static int seek_regex(compile_t* cp)
 			cp->nr++;
 		}
 	}
-	warnif(c != q, "invalid regex");
+	exerrif(cp, c != q, "bad regex");
 	cp->nc = s - cp->sc;
 	while (islower(*s)) s++;
 	cp->nc = s - cp->sc;
@@ -276,7 +284,7 @@ static int seek_string(compile_t* cp)
 			cp->nr++;
 		}
 	}
-	warnif(c != q, "invalid string");
+	exerrif(cp, c != q, "bad string");
 	cp->nc = s - cp->sc;
 	return C_string;
 }
@@ -308,17 +316,25 @@ static int seek_number(compile_t* cp)
 	cp->nc = s - cp->sc;
 	return (q ? C_float : C_int);
 }
-static int seek_symbol(compile_t* cp)
+static int seek_symbol_only(compile_t* cp)
 {
 	char *s;
-	int c, q;
-	q = !islower(*cp->sc);
+	s = cp->sc + 1;
+	while (isalnum(*s) || (*s=='_')) s++;
+	cp->nc = s - cp->sc;
+	return C_symbol;
+}
+static int seek_symbol_or_keyword(compile_t* cp)
+{
+	char *s;
+	int c;
+	c = !islower(*cp->sc);
 	s = cp->sc + 1;
 	while (isalnum(*s) || (*s=='_')) {
-		q += !islower(*s++);
+		c += !islower(*s++);
 	}
 	cp->nc = s - cp->sc;
-	if (!q) {
+	if (!c) {
 		if ((c = is_keyword(cp->sc, &cp->nc)) != 0) {
 			return c;
 		}
@@ -389,7 +405,7 @@ static int lex(compile_t* cp, int w)
 		cp->nc = s - cp->sc;
 		cp->lf |= LF_MACRO;
 		c = is_macro(cp->sc, &cp->nc);
-		warnif(!c, "unsupported macro");
+		exerrif(cp, !c, "unsupported macro");
 		return c;
 	}
 	else if ((c=='?') && (*s=='>')) {
@@ -414,7 +430,9 @@ static int lex(compile_t* cp, int w)
 		return ';';
 	}
 	else if (w & LF_RIGHT) {
-		;
+		if (isalpha(c) || (c == '_')) {
+			return seek_symbol_only(cp);
+		}
 	}
 	else {
 		if ((c == '`') || (c == '/')) {
@@ -427,7 +445,7 @@ static int lex(compile_t* cp, int w)
 			return seek_number(cp);
 		}
 		else if (isalpha(c) || (c == '_')) {
-			return seek_symbol(cp);
+			return seek_symbol_or_keyword(cp);
 		}
 	}
 	return is_operand(cp->sc, &cp->nc);
@@ -446,7 +464,7 @@ static int add_symbol(compile_t* cp, char *s, int n)
 {
 	int k, i;
 	k = strtoh(s, n);
-	warnif (k <= 0, "invalid symbol") ;
+	exerrif(cp, !k, "bad symbol") ;
 	switch (k) {
 #include "_sym2sym.h"	
 	}
@@ -454,7 +472,7 @@ static int add_symbol(compile_t* cp, char *s, int n)
 		if (cp->np[i].k == k) {
 			if ((cp->np[i].n != n)
 			|| strncmp(cp->np[i].s, s, n)) {
-				warns("%s != %s\n", cp->np[i].s, s);
+				fprintf(cp->err, "%s != %s\n", cp->np[i].s, s);
 			}
 			return k;
 		}
@@ -533,7 +551,7 @@ static void add_local(compile_t* cp, op_t c, int k, int j)
 	int i;
 	for (i = cp->fp->vi; i < cp->vn; i++) {
 		if (cp->vp[i].k == k) {
-			warns("multiple definition: %s\n", cp_symbol_to_chars(cp, k));
+			fprintf(cp->err, "multiple definition: %s\n", cp_symbol_to_chars(cp, k));
 			return;
 		}
 	}
@@ -760,32 +778,32 @@ static int expr_post(compile_t* cp)
 	vf = cp->vf;
 	c = lex(cp, LF_RIGHT); /* get right */
 	if (c == '.') {
-		c = lex(cp, 0);
-		warnif(c != C_symbol, "invalid property");
+		c = lex(cp, LF_RIGHT);
+		exerrif(cp, c != C_symbol, "bad property");
 		put_c_int(cp, C_symbol, add_symbol(cp, cp->sc, cp->nc));
 		put_c(cp, C_join);
 		cp->vf = VF_REFER;
 		c = expr_post(cp);
 	} else if (c == '[') {
 		c = expr_next(cp, 0);
-		warnif(c != ']', "invalid index");
+		exerrif(cp, c != ']', "bad index");
 		put_c(cp, C_join);
 		cp->vf = VF_REFER;
 		c = expr_post(cp);
 	} else if (c == '(') {
-		warnif(!(vf & VF_UNRESOLV), "invalid call");
+		exerrif(cp, !(vf & VF_UNRESOLV), "bad call");
 		n = 0;
 		do {
 			c = expr_next(cp, 0);
 			if (!cp->vf) break;
 			n++;
 		} while (c == ',');
-		warnif(c != ')', "invalid brace");
+		exerrif(cp, c != ')', "bad brace");
 		put_c_int(cp, C_call, n);
 		cp->vf = (vf == VF_SYMBOL) ? VF_NEW : VF_OBJECT;
 		c = expr_post(cp);
 	} else if (c==C_inc || c==C_dec) {
-		warnif(!(vf & VF_UNRESOLV), "invalid postfix");
+		exerrif(cp, !(vf & VF_UNRESOLV), "bad postfix");
 		put_c(cp, C_copy);
 		put_c(cp, C_load);
 		put_c(cp, C_sand);
@@ -813,6 +831,15 @@ static int expr_const(compile_t* cp, int k)
 {
 	int n;
 	switch (k) {
+	case K_stdin:
+		put_c(cp, C_stdin);
+		return 1;
+	case K_stdout:
+		put_c(cp, C_stdout);
+		return 1;
+	case K_stderr:
+		put_c(cp, C_stderr);
+		return 1;
 	case K___FILE__:
 		n = strlen(cp->fn);
 		put_c_int(cp, C_string, n);
@@ -912,12 +939,12 @@ static int expr_left(compile_t* cp)
 	int n;
 	c = lex(cp, 0);
 	if (c == '(') {
-		warnif( cp->vf != 0, "invalid value");
-		warnif( (c = lex(cp, 0)) <= 1, "no value" );
+		exerrif(cp,  cp->vf != 0, "bad value");
+		exerrif(cp,  (c = lex(cp, 0)) <= 1, "no value");
 		if (is_typecast(c)) {
-			warnif( lex(cp, 0) != ')', "invalid cast");
+			exerrif(cp,  lex(cp, 0) != ')', "bad cast");
 			n = expr_left(cp);
-			warnif( !cp->vf , "invalid left for cast");
+			exerrif(cp,  !cp->vf , "bad left for cast");
 			if (cp->vf & VF_UNRESOLV) {
 				put_c(cp, C_load);
 			}
@@ -929,14 +956,14 @@ static int expr_left(compile_t* cp)
 				unlex(cp, c);
 				c = expr_next(cp, -1);
 			}
-			warnif(c != ')', "invalid sub-expr");
-			warnif(!cp->vf, "invalid sub-expr");
+			exerrif(cp, c != ')', "bad sub-expr");
+			exerrif(cp, !cp->vf, "bad sub-expr");
 			c = expr_post(cp);
 		}
 	} else if (c==C_inc || c==C_dec) {
-		warnif(cp->vf != 0, "invalid prefix" );
+		exerrif(cp, cp->vf != 0, "bad prefix");
 		n = expr_left(cp);
-		warnif(!(cp->vf & VF_UNRESOLV), "invalid left for prefix");
+		exerrif(cp, !(cp->vf & VF_UNRESOLV), "bad left for prefix");
 		put_c(cp, C_copy);
 		put_c(cp, C_load);
 		put_c(cp, c);
@@ -945,28 +972,28 @@ static int expr_left(compile_t* cp)
 		cp->vf = VF_OBJECT;
 		c = n;
 	} else if (c == C_delete) {
-		warnif(cp->vf != 0, "invalid delete" );
+		exerrif(cp, cp->vf != 0, "bad delete");
 		n = expr_left(cp);
-		warnif(!(cp->vf & VF_UNRESOLV), "invalid left for delete");
+		exerrif(cp, !(cp->vf & VF_UNRESOLV), "bad left for delete");
 		put_c(cp, c);
 		cp->vf = VF_OBJECT;
 		c = n;
 	} else if (c == C_new) {
-		warnif(cp->vf != 0, "invalid new" );
+		exerrif(cp, cp->vf != 0, "bad new");
 		n = expr_left(cp);
-		warnif(cp->vf != VF_NEW, "invalid left for new");
+		exerrif(cp, cp->vf != VF_NEW, "bad left for new");
 		put_c(cp, c);
 		cp->vf = VF_OBJECT;
 		c = n;
 	} else if (c == C_bitand) { /* reference */
-		warnif(cp->vf != 0, "invalid reference" );
+		exerrif(cp, cp->vf != 0, "bad reference");
 		n = expr_left(cp);
-		warnif (!(cp->vf & VF_UNRESOLV), "invalid left for reference");
+		exerrif(cp, !(cp->vf & VF_UNRESOLV), "bad left for reference");
 		cp->vf = VF_OBJECT;
 		c = n;
 	} else if ((n = is_left(c)) > 0) {
 		c = n;
-		warnif(cp->vf != 0, "invalid prefix" );
+		exerrif(cp, cp->vf != 0, "bad prefix");
 		n = expr_left(cp);
 		if (cp->vf & VF_UNRESOLV) {
 			put_c(cp, C_load);
@@ -981,7 +1008,7 @@ static int expr_left(compile_t* cp)
 			if (!cp->vf) break;
 			n++;
 		} while (c == ',');
-		warnif(c != ']', "invalid const array");
+		exerrif(cp, c != ']', "bad const array");
 		put_c_int(cp, C_array, n);
 		cp->vf = VF_OBJECT;
 		c = lex(cp, LF_RIGHT); /* get right */
@@ -997,11 +1024,11 @@ static int expr_left(compile_t* cp)
 				break;
 			}
 			c = lex(cp, 0);
-			warnif(c != ':', "invalid item");
+			exerrif(cp, c != ':', "bad item");
 			c = expr_next(cp, 0);
 			n++;
 		} while (c == ',');
-		warnif(c != '}', "invalid const object");
+		exerrif(cp, c != '}', "bad const object");
 		put_c_int(cp, C_object, n);
 		cp->vf = VF_OBJECT;
 		c = lex(cp, LF_RIGHT); /* get right */
@@ -1034,7 +1061,7 @@ static int expr_left(compile_t* cp)
 		cp->vf = VF_CONST;
 		c = ';';
 	} else if (c == C_this) {
-		warnif(!(cp->fp->af & AF_THIS), "invalid this");
+		exerrif(cp, !(cp->fp->af & AF_THIS), "bad this");
 		put_c(cp, c);
 		cp->vf = VF_OBJECT;
 		c = expr_post(cp);
@@ -1056,13 +1083,13 @@ static int expr_next(compile_t* cp, int prev)
 		return c;
 	}
 	else if (c == '=') {
-		warnif(!(vf & VF_UNRESOLV), "invalid assign");
+		exerrif(cp, !(vf & VF_UNRESOLV), "bad assign");
 		n = expr_next(cp, 0);
 		put_c(cp, C_store);
 		c = n;
 	}
 	else if (c == C_andA || c==C_orA) {
-		warnif(!(vf & VF_UNRESOLV), "invalid assign");
+		exerrif(cp, !(vf & VF_UNRESOLV), "bad assign");
 		put_c(cp, C_copy);
 		put_c(cp, C_load);
 		put_c(cp, C_copy);
@@ -1075,7 +1102,7 @@ static int expr_next(compile_t* cp, int prev)
 		set_offset(cp, j);
 	}
 	else if ((n = is_assign(c)) != 0) {
-		warnif(!(vf & VF_UNRESOLV), "invalid assign");
+		exerrif(cp, !(vf & VF_UNRESOLV), "bad assign");
 		put_c(cp, C_copy);
 		put_c(cp, C_load);
 		c = expr_next(cp, 0);
@@ -1103,7 +1130,7 @@ static int expr_next(compile_t* cp, int prev)
 		j = seek_int(cp, 0);
 		c = expr_next(cp, 0);
 		e = seek_int(cp, 0);
-		warnif(c != ':', "invalid ?: statement");
+		exerrif(cp, c != ':', "bad ?: statement");
 		set_offset(cp, j);
 		c = expr_next(cp, 0);
 		set_offset(cp, e);
@@ -1145,7 +1172,7 @@ static int expr_part(compile_t* cp)
 	put_c(cp, C_line);
 	put_ushort(cp, cp->ln);
 	c = expr_next(cp, 0);
-	warnif(!(cp->vf & VF_RESOLV), "invalid expr");
+	exerrif(cp, !(cp->vf & VF_RESOLV), "bad expr");
 	return c;
 }
 
@@ -1155,7 +1182,7 @@ static int expr_var(compile_t* cp)
 	int k;
 	do {
 		c = lex(cp, 0);
-		warnif(c != C_symbol, "invalid define var");
+		exerrif(cp, c != C_symbol, "bad define var");
 		k = add_symbol(cp, cp->sc, cp->nc);
 		add_local(cp, C_var, k, 0);
 		c = lex(cp, 0);
@@ -1174,7 +1201,7 @@ static int expr_line(compile_t* cp)
 	op_t c;
 	do {
 		c = expr_part(cp);
-		warnif(!c || !cp->vf, "invalid expr");
+		exerrif(cp, !c || !cp->vf, "bad expr");
 		if (c == 1) {
 			put_c(cp, C_return);
 		} else if (!(cp->lf & LF_ECHO)) {
@@ -1192,7 +1219,7 @@ static int expr_line(compile_t* cp)
 				unlex(cp, c);
 				c = ';';
 			} else {
-				warns("invalid result: `%c`\n", c);
+				fprintf(cp->err, "bad result: `%c`\n", c);
 				return 0;
 			}
 		}
@@ -1301,9 +1328,9 @@ static int stmt_Minclude(compile_t* cp)
 	
 	i = cp->bn;
 	c = expr_next(cp, 0);
-	warnif(c != C_crlf, "invalid macro");
-	warnif(cp->vf != VF_CONST, "invalid include value");
-	warnif(cp->bc[i] != C_string, "invalid include value");
+	exerrif(cp, c != C_crlf, "bad macro");
+	exerrif(cp, cp->vf != VF_CONST, "bad include value");
+	exerrif(cp, cp->bc[i] != C_string, "bad include value");
 	cp->bc[i++] = C_file;
 	fn = cp->bc + i;
 	n = *(int*)fn;
@@ -1318,7 +1345,7 @@ static int stmt_Minclude(compile_t* cp)
 	}
 	
 	if (!(sc = vmopen(fn, &sz, &fp))) {
-		warns("vmopen error: %s %s\n", fn, strerror(errno));
+		fprintf(cp->err, "vmopen error: %s %s\n", fn, strerror(errno));
 		return 0;
 	}
 	
@@ -1331,6 +1358,9 @@ static int stmt_Minclude(compile_t* cp)
 	cp->fp   = old.fp;
 	cp->bc   = old.bc;
 	cp->bn   = old.bn;
+	cp->out = old.out;
+	cp->err = old.err;
+	cp->in  = old.in;
 	cp->fn   = fn;
 	cp->ln   = 1;
 	cp->ss   = sc;
@@ -1360,15 +1390,15 @@ static int stmt_Mdefine(compile_t* cp)
 	op_t c;
 	int k, i;
 	c = lex(cp, 0);
-	warnif(c != C_symbol, "invalid define");
+	exerrif(cp, c != C_symbol, "bad define");
 	k = strtoh(cp->sc, cp->nc);
 	i = cp->bn;
 	c = expr_next(cp, 0);
-	warnif(c != C_crlf, "invalid macro");
+	exerrif(cp, c != C_crlf, "bad macro");
 	if (cp->bn == i) {
 		put_c(cp, C_null);
 	} else {
-		warnif(cp->vf != VF_CONST, "invalid value for const");
+		exerrif(cp, cp->vf != VF_CONST, "bad value for const");
 	}
 	add_define(cp, k, cp->bc + i, cp->bn - i);
 	cp->bn = i;
@@ -1385,16 +1415,16 @@ static int stmt_if(compile_t* cp)
 	
 	c = lex(cp, 0);
 	
-	warnif(c != '(', "invalid if expr");
+	exerrif(cp, c != '(', "bad if expr");
 	
 	c = expr_part(cp);
 	
-	warnif(c != ')', "invalid if expr");
+	exerrif(cp, c != ')', "bad if expr");
 	
 	put_c(cp, C_and);
 	j = seek_int(cp, 0);
 	c = stmt_line(cp);
-	warnif(c != ';', "invalid if statement");
+	exerrif(cp, c != ';', "bad if statement");
 	
 	c = lex(cp, LF_LEFT);
 	if (c == C_else) {
@@ -1422,7 +1452,7 @@ static int stmt_foreach(compile_t* cp)
 	frame_in(cp, &fp, AF_BREAK | AF_CONTINUE);
 	
 	c = lex(cp, 0);
-	warnif(c != '(', "invalid foreach expr");
+	exerrif(cp, c != '(', "bad foreach expr");
 	
 	c = lex(cp, 0);
 	d = 0; /* define or not */
@@ -1430,7 +1460,7 @@ static int stmt_foreach(compile_t* cp)
 		c = lex(cp, 0);
 		d = 1;
 	}
-	warnif(c != C_symbol, "invalid symbol");
+	exerrif(cp, c != C_symbol, "bad symbol");
 	k = 0;
 	v = add_symbol(cp, cp->sc, cp->nc);
 	if (d) {
@@ -1439,7 +1469,7 @@ static int stmt_foreach(compile_t* cp)
 	c = lex(cp, 0);
 	if (c == ',') {
 		c = lex(cp, 0);
-		warnif(c != C_symbol, "invalid symbol");
+		exerrif(cp, c != C_symbol, "bad symbol");
 		k = v;
 		v = add_symbol(cp, cp->sc, cp->nc);
 		if (d) {
@@ -1447,14 +1477,14 @@ static int stmt_foreach(compile_t* cp)
 		}
 		c = lex(cp, 0);
 	}
-	warnif(c != C_in, "not 'IN'");
+	exerrif(cp, c != C_in, "not 'IN'");
 	
 	put_c_int(cp, C_symbol, k);
 	put_c_int(cp, C_symbol, v);
 	
 	c = expr_part(cp);
 	
-	warnif(c != ')', "invalid foreach expr");
+	exerrif(cp, c != ')', "bad foreach expr");
 	
 	s = cp->bn;
 	add_local(cp, C_label, K_continue, cp->bn);
@@ -1486,9 +1516,9 @@ static int stmt_for(compile_t* cp)
 	frame_in(cp, &fp, AF_BREAK | AF_CONTINUE);
 	
 	c = lex(cp, 0);
-	warnif(c != '(', "invalid for expr");
+	exerrif(cp, c != '(', "bad for expr");
 	
-	warnif((c = lex(cp, 0)) <= 1, "invalid for expr");
+	exerrif(cp, (c = lex(cp, 0)) <= 1, "bad for expr");
 	if (c == C_var) {
 		c = expr_var(cp);
 	} else {
@@ -1497,16 +1527,16 @@ static int stmt_for(compile_t* cp)
 			c = expr_line(cp);
 		}
 	}
-	warnif(c != ';', "invalid for expr");
+	exerrif(cp, c != ';', "bad for expr");
 	
 	r = s = cp->bn;
 	c = expr_part(cp);
 	
-	warnif(c != ';', "invalid for expr");
+	exerrif(cp, c != ';', "bad for expr");
 	put_c(cp, C_loop);
 	e = seek_int(cp, 0);
 	
-	warnif((c = lex(cp, 0)) <= 1, "invalid for expr");
+	exerrif(cp, (c = lex(cp, 0)) <= 1, "bad for expr");
 	if (c != ')') {
 		unlex(cp, c);
 		
@@ -1515,7 +1545,7 @@ static int stmt_for(compile_t* cp)
 		
 		r = cp->bn;
 		c = expr_line(cp);
-		warnif(c != ')', "invalid for expr");
+		exerrif(cp, c != ')', "bad for expr");
 		
 		put_c(cp, C_jump);
 		put_offset(cp, s);
@@ -1546,11 +1576,11 @@ static int stmt_while(compile_t* cp)
 	s = cp->bn;
 	add_local(cp, C_label, K_continue, cp->bn);
 	c = lex(cp, 0);
-	warnif(c != '(', "invalid while expr");
+	exerrif(cp, c != '(', "bad while expr");
 	
 	c = expr_part(cp);
 	
-	warnif(c != ')', "invalid whiel expr");
+	exerrif(cp, c != ')', "bad whiel expr");
 	
 	put_c(cp, C_loop);
 	e = seek_int(cp, 0);
@@ -1577,19 +1607,19 @@ static int stmt_do(compile_t* cp)
 	
 	s = cp->bn;
 	c = stmt_line(cp);
-	warnif(c != ';', "invalid do");
+	exerrif(cp, c != ';', "bad do");
 	
 	c = lex(cp, 0);
-	warnif(c != C_while, "invalid do");
+	exerrif(cp, c != C_while, "bad do");
 	
 	c = lex(cp, 0);
-	warnif(c != '(', "invalid while expr");
+	exerrif(cp, c != '(', "bad while expr");
 	
 	add_local(cp, C_label, K_continue, cp->bn);
 	
 	c = expr_part(cp);
 	
-	warnif(c != ')', "invalid while expr");
+	exerrif(cp, c != ')', "bad while expr");
 	
 	put_c(cp, C_loop);
 	e = seek_int(cp, 0);
@@ -1615,17 +1645,17 @@ static int stmt_switch(compile_t* cp)
 	frame_in(cp, &fp, AF_BREAK | AF_BLOCK);
 	
 	c = lex(cp, 0);
-	warnif(c != '(', "invalid switch expr");
+	exerrif(cp, c != '(', "bad switch expr");
 	
 	c = expr_part(cp);
 	
-	warnif(c != ')', "invalid switch expr");
+	exerrif(cp, c != ')', "bad switch expr");
 	
-	warnif(lex(cp, LF_LEFT) != '{', "invalid switch statement");
+	exerrif(cp, lex(cp, LF_LEFT) != '{', "bad switch statement");
 	
 	j = k = 0;
 	do {
-		warnif((c = lex(cp, LF_LEFT)) <= 1, "invalid switch content");
+		exerrif(cp, (c = lex(cp, LF_LEFT)) <= 1, "bad switch content");
 		if (c == C_case) {
 			if (j) {
 				put_c(cp, C_jump);
@@ -1633,13 +1663,13 @@ static int stmt_switch(compile_t* cp)
 				set_offset(cp, j);
 			}
 			c = expr_part(cp);
-			warnif(c != ':', "invalid case");
+			exerrif(cp, c != ':', "bad case");
 			put_c(cp, C_case);
 			j = seek_int(cp, 0);
 			c = ';';
 		} else if (c == C_default) {
 			c = lex(cp, 0);
-			warnif(c != ':', "invalid default");
+			exerrif(cp, c != ':', "bad default");
 			if (j) set_offset(cp, j);
 			if (k) set_offset(cp, k);
 			j = k = 0;
@@ -1655,7 +1685,7 @@ static int stmt_switch(compile_t* cp)
 		}
 	} while (c == ';');
 	
-	warnif(c != '}', "invalid switch statement");
+	exerrif(cp, c != '}', "bad switch statement");
 	
 	if (j) set_offset(cp, j);
 	if (k) set_offset(cp, k);
@@ -1676,21 +1706,23 @@ static int stmt_try(compile_t* cp)
 	
 	frame_in(cp, &fp, AF_THROW);
 	
+	put_c(cp, C_tmperr);
+	
 	c = stmt_line(cp);
-	warnif(c != ';', "invalid try statement");
+	exerrif(cp, c != ';', "bad try statement");
 	
 	c = lex(cp, LF_LEFT);
 	if (c == C_catch) {
 		put_c(cp, C_jump);
 		j = seek_int(cp, 0);
-		add_local(cp, C_label, K_throw, cp->bn);
+		add_local(cp, C_label, K_catch, cp->bn);
 		c = lex(cp, 0);
-		warnif(c != '(', "invalid catch arg");
+		exerrif(cp, c != '(', "bad catch arg");
 		c = lex(cp, 0);
-		warnif(c != C_symbol, "invalid symbol");
+		exerrif(cp, c != C_symbol, "bad symbol");
 		put_c_int(cp, C_catch, add_symbol(cp, cp->sc, cp->nc));
 		c = lex(cp, 0);
-		warnif(c != ')', "invalid catch expr");
+		exerrif(cp, c != ')', "bad catch expr");
 		c = stmt_line(cp);
 		set_offset(cp, j);
 	} else {
@@ -1698,7 +1730,7 @@ static int stmt_try(compile_t* cp)
 			unlex(cp, c);
 			c = ';';
 		}
-		add_local(cp, C_label, K_throw, cp->bn);
+		add_local(cp, C_label, K_catch, cp->bn);
 		put_c_int(cp, C_catch, 0);
 	}
 	
@@ -1717,7 +1749,7 @@ static int stmt_function(compile_t* cp)
 	j = seek_int(cp, 0);
 	
 	c = lex(cp, 0);
-	warnif(c != C_symbol, "invalid function symbol");
+	exerrif(cp, c != C_symbol, "bad function symbol");
 	
 	k = add_symbol(cp, cp->sc, cp->nc);
 	add_local(cp, C_function, k, cp->bn);
@@ -1725,7 +1757,7 @@ static int stmt_function(compile_t* cp)
 	frame_in(cp, &fp, AF_FILE | AF_RETURN);
 	
 	c = lex(cp, 0);
-	warnif(c != '(', "invalid function args");
+	exerrif(cp, c != '(', "bad function args");
 	
 	n = 0;
 	do {
@@ -1738,11 +1770,11 @@ static int stmt_function(compile_t* cp)
 			c = lex(cp, 0);
 		}
 	} while (c == ',');
-	warnif(c != ')', "invalid function args");
+	exerrif(cp, c != ')', "bad function args");
 	
 	put_c_int(cp, C_args, n);
 	
-	warnif((c = lex(cp, LF_LEFT)) != '{', "invalid function statement");
+	exerrif(cp, (c = lex(cp, LF_LEFT)) != '{', "bad function statement");
 	unlex(cp, c);
 	
 	c = stmt_line(cp);
@@ -1766,7 +1798,7 @@ static int stmt_class(compile_t* cp)
 	j = seek_int(cp, 0);
 	
 	c = lex(cp, 0);
-	warnif(c != C_symbol, "invalid class symbol");
+	exerrif(cp, c != C_symbol, "bad class symbol");
 	
 	k = add_symbol(cp, cp->sc, cp->nc);
 	add_local(cp, C_class, k, cp->bn);
@@ -1788,14 +1820,14 @@ static int stmt_class(compile_t* cp)
 				c = lex(cp, 0);
 			}
 		} while (c == ',');
-		warnif(c != ')', "invalid cass args");
+		exerrif(cp, c != ')', "bad cass args");
 		
 		put_c_int(cp, C_args, n);
 		
 		c = lex(cp, LF_LEFT);
 	}
 	
-	warnif(c != '{', "invalid class statement");
+	exerrif(cp, c != '{', "bad class statement");
 	unlex(cp, c);
 	
 	c = stmt_line(cp);
@@ -1824,7 +1856,7 @@ static int stmt_block(compile_t* cp)
 	do {
 		c = stmt_line(cp);
 	} while (c == ';');
-	warnif(c != '}', "invalid block");
+	exerrif(cp, c != '}', "bad block");
 	
 	if (af) {
 		cp->fp->af = af;
@@ -1858,68 +1890,68 @@ static int stmt_line(compile_t* cp)
 	} else if (c == C_try) {
 		c = stmt_try(cp);
 	} else if (c == C_class) {
-		warnif(!(cp->fp->af & AF_CLASS), "invalid class");
+		exerrif(cp, !(cp->fp->af & AF_CLASS), "bad class");
 		c = stmt_class(cp);
 	} else if (c == C_function) {
-		warnif(!(cp->fp->af & AF_FUNCTION), "invalid function");
+		exerrif(cp, !(cp->fp->af & AF_FUNCTION), "bad function");
 		c = stmt_function(cp);
 	} else if (c == C_break) {
-		warnif(!(cp->fp->af & AF_BREAK), "invalid break");
+		exerrif(cp, !(cp->fp->af & AF_BREAK), "bad break");
 		put_c_int(cp, C_goto, K_break);
 		c = lex(cp, 0);
-		warnif(c != ';', "invalid break");
+		exerrif(cp, c != ';', "bad break");
 	} else if (c == C_continue) {
-		warnif(!(cp->fp->af & AF_CONTINUE), "invalid continue");
+		exerrif(cp, !(cp->fp->af & AF_CONTINUE), "bad continue");
 		put_c_int(cp, C_goto, K_continue);
 		c = lex(cp, 0);
-		warnif(c != ';', "invalid continue");
+		exerrif(cp, c != ';', "bad continue");
 	} else if (c == C_return) {
-		warnif(!(cp->fp->af & AF_RETURN), "invalid return");
-		warnif((c = lex(cp, 0)) <= 1, "invalid return");
+		exerrif(cp, !(cp->fp->af & AF_RETURN), "bad return");
+		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad return");
 		if (c == ';') {
 			put_c(cp, C_null);
 		} else {
 			unlex(cp, c);
 			c = expr_part(cp);
 		}
-		warnif(c != ';', "invalid return");
+		exerrif(cp, c != ';', "bad return");
 		put_c(cp, C_return);
 	} else if (c == C_throw) {
-		warnif((c = lex(cp, 0)) <= 1, "invalid throw");
+		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad throw");
 		if (c == ';') {
 			put_c(cp, C_null);
 		} else {
 			unlex(cp, c);
 			c = expr_part(cp);
 		}
-		warnif(c != ';', "invalid throw");
+		exerrif(cp, c != ';', "bad throw");
 		put_c(cp, C_throw);
 	} else if (c == C_exit) {
-		warnif((c = lex(cp, 0)) <= 1, "invalid exit");
+		exerrif(cp, (c = lex(cp, 0)) <= 1, "bad exit");
 		if (c == ';') {
 			put_c(cp, C_null);
 		} else {
 			unlex(cp, c);
 			c = expr_part(cp);
 		}
-		warnif(c != ';', "invalid exit");
+		exerrif(cp, c != ';', "bad exit");
 		put_c(cp, C_exit);
 	} else if (c == C_goto) {
 		c = lex(cp, 0);
-		warnif(c != C_symbol, "invalid label");
+		exerrif(cp, c != C_symbol, "bad label");
 		put_c_int(cp, C_goto, add_symbol(cp, cp->sc, cp->nc));
 		c = lex(cp, 0);
-		warnif(c != ';', "invalid goto");
+		exerrif(cp, c != ';', "bad goto");
 	} else if (c == C_Minclude) {
 		c = stmt_Minclude(cp);
 	} else if (c == C_Mdefine) {
 		c = stmt_Mdefine(cp);
 	} else if (c == C_label) {
 		c = lex(cp, 0);
-		warnif(c != C_symbol, "invalid label");
+		exerrif(cp, c != C_symbol, "bad label");
 		add_local(cp, C_label, add_symbol(cp, cp->sc, cp->nc), cp->bn);
 		c = lex(cp, 0);
-		warnif(c != ':', "invalid label");
+		exerrif(cp, c != ':', "bad label");
 		c = ';';
 	} else if (c == C_var) {
 		c = expr_var(cp);
@@ -1935,27 +1967,6 @@ static int stmt_line(compile_t* cp)
 /************************************************************
  *
  ************************************************************/
-static void cp_trace(compile_t* cp)
-{
-	char *s, *e;
-	
-	s = e = cp->sc;
-	
-	while (((cp->sc - s) < 16) && (s > cp->ss)
-	&& (*(s-1) != 0) && !iscrlf(*(s-1))) { s--; }
-	
-	while (((e - cp->sc) < 16) && (e < cp->es)
-	&& (*(e+1) != 0) && !iscrlf(*(e+1))) { e++; }
-	
-	fprintf(g_stdout, "%.*s>%.*s",
-		(cp->sc - s), s, (e - cp->sc), cp->sc);
-	
-	if (cp->fn) {
-		fprintf(g_stdout, " [%s:%d]", cp->fn, cp->ln);
-	}
-	fputc('\n', g_stdout);
-}
-
 static int stmt_file(compile_t* cp, cp_frame_t* fp)
 {
 	char *sc;
@@ -1998,7 +2009,15 @@ static int stmt_file(compile_t* cp, cp_frame_t* fp)
 	} while (c == ';');
 	
 	if (c != 1) {
-		cp_trace(cp);
+		char *s, *e;
+		s = e = cp->sc;
+		while (((cp->sc - s) < 16) && (s > cp->ss)
+			&& (*(s-1) != 0) && !iscrlf(*(s-1))) { s--; }
+		while (((e - cp->sc) < 16) && (e < cp->es)
+			&& (*(e+1) != 0) && !iscrlf(*(e+1))) { e++; }
+		fprintf(cp->err, "%.*s>%.*s [%s:%d]\n",
+			(int)(cp->sc - s), s, (int)(e - cp->sc), cp->sc,
+			cp->fn, cp->ln);
 	}
 	
 	if (fp) {
@@ -2011,7 +2030,7 @@ static int stmt_file(compile_t* cp, cp_frame_t* fp)
 /************************************************************
  *
  ************************************************************/
-int compile_code_to_bytes(char* fn, char *sc, int sz, char **bc, int* bn)
+int compile_code(exec_t* ex, char* fn, char *sc, int sz, char **bc, int* bn)
 {
 	compile_t cp;
 	cp_frame_t fp;
@@ -2025,6 +2044,9 @@ int compile_code_to_bytes(char* fn, char *sc, int sz, char **bc, int* bn)
 	cp.ss = sc;
 	cp.sc = sc;
 	cp.es = sc + sz;
+	cp.out = ex->out;
+	cp.err = ex->err;
+	cp.in  = ex->in;
 	
 	r = 0;
 	if (!stmt_file(&cp, &fp)) {
@@ -2037,25 +2059,25 @@ int compile_code_to_bytes(char* fn, char *sc, int sz, char **bc, int* bn)
 	return r;
 }
 
-int compile_file_to_bytes(char *fn, char **bc, int* bn)
+int compile_file(exec_t* ex, char *fn, char **bc, int* bn)
 {
 	FILE *fp;
 	char* sc;
 	int sz, r;
 	
 	if (!(sc = vmopen(fn, &sz, &fp))) {
-		warns("vmopen error: %s %s\n", fn, strerror(errno));
+		fprintf(ex->err, "vmopen error: %s %s\n", fn, strerror(errno));
 		return -1;
 	}
 	
-	r = compile_code_to_bytes(fn, sc, sz, bc, bn);
+	r = compile_code(ex, fn, sc, sz, bc, bn);
 	
 	vmclose(sc, sz, fp);
 	
 	return r;
 }
 
-int compile_file_to_file(char *scfile, char *bcfile)
+int compile_file_to_file(exec_t* ex, char *infn, char *outfn)
 {
 	FILE* fp;
 	char *bc;
@@ -2065,20 +2087,20 @@ int compile_file_to_file(char *scfile, char *bcfile)
 	bc = NULL;
 	bn = 0;
 	
-	r = compile_file_to_bytes(scfile, &bc, &bn);
+	r = compile_file(ex, infn, &bc, &bn);
 	if (r != 0) {
 		if (bc) free(bc);
 		return r;
 	}
 	
-	fp = fopen(bcfile, "w");
+	fp = fopen(outfn, "w");
 	if (!fp) {
-		warns("fopen error: %s %s\n", bcfile, strerror(errno));
+		fprintf(ex->err, "fopen error: %s %s\n", outfn, strerror(errno));
 	} else {
 		fputs("#!/usr/bin/js\n", fp);
 		fwrite(bc, 1, bn, fp);
 		fclose(fp);
-		r = chmod(bcfile, 0755);
+		r = chmod(outfn, 0755);
 	}
 	
 	free(bc);
