@@ -55,44 +55,46 @@ static char* read_offset(char **p)
 /************************************************************
  *
  ************************************************************/
-static int ex_sp_push(exec_t* ex, var_t* v)
+static int push_stack(exec_t* ex, var_t* v)
 {
 	return array_push(ex, ex->sp, v);
 }
-static int ex_sp_pop(exec_t* ex, var_t* v)
+static int pop_stack(exec_t* ex, var_t* v)
 {
 	return array_pop(ex, ex->sp, v);
 }
-static int ex_sp_get(exec_t* ex, int i, var_t* v)
+static int load_stack(exec_t* ex, int i, var_t* v)
 {
 	return array_get(ex, ex->sp, i, v);
 }
-static int ex_set_res(exec_t* ex, var_t* v)
+static int store_result(exec_t* ex, var_t* v)
 {
-	if (ex->res->type) {
+	if (ex->res->p) {
 		delete_var(ex, ex->res);
 	}
 	copy_var(ex->res, v);
 	return 1;
 }
-static int ex_del_res(exec_t* ex, var_t* v)
+static int delete_result(exec_t* ex, var_t* v)
 {
-	if (!ex->res->type) {
-		v->type = 0;
+	if (!ex->res->p) {
+		v->p = NULL;
 	} else {
 		copy_var(v, ex->res);
-		ex->res->type = 0;
+		ex->res->p = NULL;
 	}
 	return 1;
 }
-static int ex_on_goto(exec_t* ex)
+static int goto_label(exec_t* ex)
 {
 	var_t v;
-	if (!frame_getLocal(ex, ex->fp, ex->req, &v)) {
+	if (!frame_getLocal(ex, ex->fp, ex->jmp, &v)) {
 		return 0;
 	}
-	ex->req = 0;
-	errif(v.type != V_label, "invalid type");
+	ex->jmp = 0;
+	
+	exerrif(ex, v.p != label_proto, "bad type");
+	
 	ex->bc = v.u.p;
 	return 1;
 }
@@ -102,17 +104,17 @@ static int ex_on_goto(exec_t* ex)
  ************************************************************/
 static int ex_void(exec_t* ex)
 {
-	return 1;
+	return -1;
 }
 static int ex_file(exec_t* ex)
 {
 	read_chars(&ex->bc, &ex->file);
-	return 1;
+	return -1;
 }
 static int ex_line(exec_t* ex)
 {
 	ex->line = read_ushort(&ex->bc);
-	return 1;
+	return -1;
 }
 static int ex_names(exec_t* ex)
 {
@@ -121,9 +123,9 @@ static int ex_names(exec_t* ex)
 	n = read_int(&ex->bc);
 	while (n-- > 0) {
 		m = read_chars(&ex->bc, &s);
-		k = string_to_symbol(s, m);
+		k = string_to_symbol(ex, s, m);
 	}
-	return 1;
+	return -1;
 }
 static int ex_var(exec_t* ex)
 {
@@ -132,7 +134,7 @@ static int ex_var(exec_t* ex)
 	k = read_int(&ex->bc);
 	new_null(&v);
 	frame_setLocal(ex, ex->fp, k, &v);
-	return 1;
+	return -1;
 }
 static int ex_label(exec_t* ex)
 {
@@ -143,7 +145,7 @@ static int ex_label(exec_t* ex)
 	j = read_offset(&ex->bc);
 	new_label(&v, j);
 	frame_setLocal(ex, ex->fp, k, &v);
-	return 1;
+	return -1;
 }
 
 /************************************************************
@@ -154,29 +156,29 @@ static int ex_jump(exec_t* ex)
 	char *j;
 	j = read_offset(&ex->bc);
 	ex->bc = j;
-	return 1;
+	return -1;
 }
 static int ex_and(exec_t* ex)
 {
 	var_t v;
 	char* j;
 	j = read_offset(&ex->bc);
-	ex_sp_pop(ex, &v);
-	if (var_toBool(&v) == 0) {
+	pop_stack(ex, &v);
+	if (var_toBool(ex, &v) == 0) {
 		ex->bc = j;
 	}
-	return 1;
+	return -1;
 }
 static int ex_or(exec_t* ex)
 {
 	var_t v;
 	char* j;
 	j = read_offset(&ex->bc);
-	ex_sp_pop(ex, &v);
-	if (var_toBool(&v) != 0) {
+	pop_stack(ex, &v);
+	if (var_toBool(ex, &v) != 0) {
 		ex->bc = j;
 	}
-	return 1;
+	return -1;
 }
 static int ex_case(exec_t* ex)
 {
@@ -185,14 +187,14 @@ static int ex_case(exec_t* ex)
 	
 	j = read_offset(&ex->bc);
 	
-	ex_sp_pop(ex, &v2);
-	ex_sp_get(ex, -1, &v1);
+	pop_stack(ex, &v2);
+	load_stack(ex, -1, &v1);
 	
-	if (var_cmp(&v1, &v2) != 0) {
+	if (cmp_vars(ex, &v1, &v2) != 0) {
 		ex->bc = j;
 	}
 	
-	return 1;
+	return -1;
 }
 static int ex_loop(exec_t* ex)
 {
@@ -202,16 +204,16 @@ static int ex_loop(exec_t* ex)
 	
 	e = read_offset(&ex->bc);
 	
-	ex_sp_pop(ex, &v);
+	pop_stack(ex, &v);
 	
 	i = ex->lc++;
 	
-	errif(i < 0, "out of loop range");
+	exerrif(ex, i < 0, "out of loop range");
 	
-	if (var_toBool(&v) == 0) {
+	if (var_toBool(ex, &v) == 0) {
 		ex->bc = e;
 	}
-	return 1;
+	return -1;
 }
 static int ex_in(exec_t* ex)
 {
@@ -227,19 +229,19 @@ static int ex_in(exec_t* ex)
 	
 	i = ex->lc++;
 	
-	errif(i < 0, "out of loop range");
+	exerrif(ex, i < 0, "out of loop range");
 	
 	if (var_eachItem(ex, &v1, i, &v2, &v3) <= 0) {
 		ex->bc = e;
 	} else {
-		if (k > 0) {
+		if (k != 0) {
 			frame_set(ex, ex->fp, k, &v2);
 		}
-		if (j > 0) {
+		if (j != 0) {
 			frame_set(ex, ex->fp, j, &v3);
 		}
 	}
-	return 1;
+	return -1;
 }
 
 /************************************************************
@@ -250,29 +252,44 @@ static int ex_args(exec_t* ex)
 	var_t v;
 	int i, j, n;
 	n = read_int(&ex->bc);
-	errif(ex->argc < n, "invalid arguments");
+	
+	exerrif(ex, ex->argc < n, "bad arguments");
+	
 	for (i = -n, j = 0; (i < 0) && (j < ex->argc); i++, j++) {
-		ex_sp_get(ex, i, &v);
-		errif(v.type != V_symbol, "invalid type");
+		load_stack(ex, i, &v);
+		
+		exerrif(ex, v.p != symbol_proto, "bad type");
+		
 		frame_setLocal(ex, ex->fp, v.u.i, ex->argv + j);
 	}
 	while (n-- > 0) {
-		ex_sp_pop(ex, &v);
+		pop_stack(ex, &v);
 	}
-	return 1;
+	return -1;
 }
 static int ex_catch(exec_t* ex)
 {
 	var_t v;
 	int k;
 	k = read_int(&ex->bc);
-	ex_del_res(ex, &v);
+	delete_result(ex, &v);
 	if (!k) {
 		delete_var(ex, &v);
 	} else {
+		if (!v.p) {
+			FILE *fp;
+			char *s;
+			int n;
+			fp = ex->err;
+			n = ftell(fp);
+			rewind(fp);
+			s = calloc(n + 1, 1);
+			fread(s, 1, n, fp);
+			new_string_with(ex, &v, s, n);
+		}
 		frame_setLocal(ex, ex->fp, k, &v);
 	}
-	return 1;
+	return -1;
 }
 
 /************************************************************
@@ -280,36 +297,38 @@ static int ex_catch(exec_t* ex)
  ************************************************************/
 static int ex_goto(exec_t* ex)
 {
-	ex->req = read_int(&ex->bc);
-	if (ex_on_goto(ex)) {
-		return 1;
+	ex->jmp = read_int(&ex->bc);
+	if (goto_label(ex)) {
+		return -1; /* continue */
 	}
-	return K_goto;
+	return 0; /* no result */
 }
 static int ex_throw(exec_t* ex)
 {
 	var_t v;
-	ex_sp_pop(ex, &v);
-	ex_set_res(ex, &v);
-	ex->req = K_throw;
-	if (ex_on_goto(ex)) {
-		return 1;
+	pop_stack(ex, &v);
+	store_result(ex, &v);
+	ex->jmp = K_catch;
+	if (goto_label(ex)) {
+		return -1; /* continue */
 	}
-	return K_throw;
+	return 1; /* has result */
 }
 static int ex_exit(exec_t* ex)
 {
 	var_t v;
-	ex_sp_pop(ex, &v);
-	ex_set_res(ex, &v);
-	return K_exit;
+	pop_stack(ex, &v);
+	store_result(ex, &v);
+	ex->jmp = K_exit;
+	return 1; /* has result */
 }
 static int ex_return(exec_t* ex)
 {
 	var_t v;
-	ex_sp_pop(ex, &v);
-	ex_set_res(ex, &v);
-	return K_return;
+	pop_stack(ex, &v);
+	store_result(ex, &v);
+	ex->jmp = K_return;
+	return 1; /* has result */
 }
 
 /************************************************************
@@ -317,7 +336,8 @@ static int ex_return(exec_t* ex)
  ************************************************************/
 static int ex_frameOut(exec_t* ex)
 {
-	return K_break;
+	ex->jmp = K_break;
+	return 0; /* no result */
 }
 static int ex_frameIn(exec_t* ex)
 {
@@ -325,18 +345,22 @@ static int ex_frameIn(exec_t* ex)
 	
 	r = call_frame(ex);
 	
-	switch (r) {
-	case -1:
-	case K_throw:
-		ex->req = K_throw;
-	case K_goto:
-		assert(ex->req != 0);
-		if (ex_on_goto(ex)) {
-			r = 1;
-		}
-		break;
+	switch (ex->jmp) {
 	case K_break:
-		r = 1;
+		assert(r == 0);
+		ex->jmp = 0;
+	case 0:
+		r = -1;
+		break;
+	case K_exit:
+	case K_return:
+		assert(r == 1);
+		break;
+	case K_catch:
+	default:
+		if (goto_label(ex)) {
+			r = -1;
+		}
 		break;
 	}
 	
@@ -349,35 +373,42 @@ static int ex_call(exec_t* ex)
 	int i, r, argc;
 	
 	argc = read_int(&ex->bc);
-	errif(argc >= 256, "too large arguments");
+	
+	exerrif(ex, argc >= 256, "too large arguments");
+	
 	for (i = argc; i > 0; i--) {
-		ex_sp_pop(ex, &ref);
+		pop_stack(ex, &ref);
 	}
 	argv = ex->sp->v + ex->sp->n;
-	ex_sp_pop(ex, &ref);
+	pop_stack(ex, &ref);
 	
-	res.type = 0;
+	res.p = NULL;
 	
 	r = refer_call(ex, &ref, argc, argv, &res);
 	
-	switch (r) {
-	case -1:
-	case K_throw:
-		ex->req = K_throw;
-	case K_goto:
-		assert(ex->req != 0);
-		ex_set_res(ex, &res);
-		if (ex_on_goto(ex)) {
-			r = 1;
-		}
+	switch (ex->jmp) {
+	case K_return:
+		assert(r == 1);
+		ex->jmp = 0;
+	case 0:
+		push_stack(ex, &res);
+		r = -1;  /* continue */
 		break;
 	case K_exit:
-		ex_set_res(ex, &res);
+		assert(r == 1);
+		store_result(ex, &res);
 		break;
-	case K_return:
+	case K_catch:
+		if (r) {
+			/* throwed */
+			store_result(ex, &res);
+		} else {
+			/* unknown function */
+		}
 	default:
-		ex_sp_push(ex, &res);
-		r = 1;
+		if (goto_label(ex)) {
+			r = -1;
+		}
 		break;
 	}
 	
@@ -394,30 +425,37 @@ static int ex_class(exec_t* ex)
 	var_t v;
 	int k;
 	char* j;
+	
 	k = read_int(&ex->bc);
 	j = read_offset(&ex->bc);
 	new_class(&v, j);
 	frame_setLocal(ex, ex->gp, k, &v);
-	return 1;
+	
+	return -1;
 }
 static int ex_function(exec_t* ex)
 {
 	var_t v;
 	int k;
 	char* j;
+	
 	k = read_int(&ex->bc);
 	j = read_offset(&ex->bc);
 	new_function(&v, j);
 	frame_setLocal(ex, ex->gp, k, &v);
-	return 1;
+	
+	return -1;
 }
 static int ex_this(exec_t* ex)
 {
 	var_t v;
-	errif( !ex->gp->p , "this global has not parent");
+	
+	exerrif(ex,  !ex->gp->p , "this global has not parent");
+	
 	new_frame(ex, &v, ex->gp);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	
+	return -1;
 }
 
 /************************************************************
@@ -427,50 +465,50 @@ static int ex_store(exec_t* ex)
 {
 	var_t ref, val;
 	
-	ex_sp_pop(ex, &val);
-	ex_sp_get(ex, -1, &ref);
+	pop_stack(ex, &val);
+	load_stack(ex, -1, &ref);
 	
 	if (!refer_set(ex, &ref, &val)) {
 		;
 	}
-	return 1;
+	return -1;
 }
 static int ex_load(exec_t* ex)
 {
 	var_t ref, res;
 	
-	ex_sp_pop(ex, &ref);
+	pop_stack(ex, &ref);
 	
 	if (!refer_get(ex, &ref, &res)) {
 		new_null(&res);
 	}
-	ex_sp_push(ex, &res);
+	push_stack(ex, &res);
 	
 	delete_var(ex, &ref);
-	return 1;
+	return -1;
 }
 static int ex_delete(exec_t* ex)
 {
 	var_t ref, res;
 	
-	ex_sp_pop(ex, &ref);
+	pop_stack(ex, &ref);
 	if (!refer_del(ex, &ref, &res)) {
 		new_null(&res);
 	}
 	
-	ex_sp_push(ex, &res);
+	push_stack(ex, &res);
 	
 	delete_var(ex, &ref);
-	return 1;
+	return -1;
 }
 static int ex_join(exec_t* ex)
 {
 	var_t v1, v2, v3, v4;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	if (v1.type != V_refer) {
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	if (v1.p != refer_proto) {
 		new_refer(ex, &v3, 0);
-		if (v1.type == V_symbol) {
+		if (v1.p == symbol_proto) {
 			new_frame(ex, &v4, ex->fp);
 			array_push(ex, v3.u.p, &v4);
 		}
@@ -478,12 +516,12 @@ static int ex_join(exec_t* ex)
 		copy_var(&v1, &v3);
 	}
 	array_push(ex, v1.u.p, &v2);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_new(exec_t* ex)
 {
-	return 1;
+	return -1;
 }
 
 /************************************************************
@@ -495,22 +533,22 @@ static int ex_sand(exec_t* ex)
 	copy_var(&v, ex->sp->v + (ex->sp->n - 1));
 	copy_var(ex->sp->v + (ex->sp->n - 1), ex->sp->v + (ex->sp->n - 2));
 	copy_var(ex->sp->v + (ex->sp->n - 2), &v);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_copy(exec_t* ex)
 {
 	var_t v;
-	ex_sp_get(ex, -1, &v);
-	ex_sp_push(ex, &v);
-	return 1;
+	load_stack(ex, -1, &v);
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_pop(exec_t* ex)
 {
 	var_t v;
-	ex_sp_pop(ex, &v);
+	pop_stack(ex, &v);
 	delete_var(ex, &v);
-	return 1;
+	return -1;
 }
 static int ex_echo(exec_t* ex)
 {
@@ -518,23 +556,23 @@ static int ex_echo(exec_t* ex)
 	char *s;
 	int c, n;
 	c = *ex->bc++;
-	ex_sp_pop(ex, &v);
-	if (var_toString(&v, &s, &n)) {
-		fwrite(s, 1, n, g_stdout);
+	pop_stack(ex, &v);
+	if (var_toString(ex, &v, &s, &n)) {
+		fwrite(s, 1, n, ex->out);
 	}
 	if (c) {
-		fputc(c, g_stdout);
+		fputc(c, ex->out);
 	}
 	delete_var(ex, &v);
-	return 1;
+	return -1;
 }
 static int ex_doc(exec_t* ex)
 {
 	char *s;
 	int n;
 	n = read_string(&ex->bc, &s);
-	fwrite(s, 1, n, g_stdout);
-	return 1;
+	fwrite(s, 1, n, ex->out);
+	return -1;
 }
 
 /************************************************************
@@ -547,8 +585,8 @@ static int ex_string(exec_t* ex)
 	int n;
 	n = read_string(&ex->bc, &s);
 	new_string_copy(ex, &v, s, n);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_regex(exec_t* ex)
 {
@@ -558,8 +596,8 @@ static int ex_regex(exec_t* ex)
 	i = read_int(&ex->bc);
 	n = read_chars(&ex->bc, &s);
 	new_regex(ex, &v, s, n, i);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_char(exec_t* ex)
 {
@@ -567,8 +605,8 @@ static int ex_char(exec_t* ex)
 	int c;
 	c = read_int(&ex->bc);
 	new_char(&v, c);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_int(exec_t* ex)
 {
@@ -576,8 +614,8 @@ static int ex_int(exec_t* ex)
 	int i;
 	i = read_int(&ex->bc);
 	new_int(&v, i);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_float(exec_t* ex)
 {
@@ -585,29 +623,29 @@ static int ex_float(exec_t* ex)
 	double f;
 	f = read_float(&ex->bc);
 	new_float(&v, f);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_true(exec_t* ex)
 {
 	var_t v;
 	new_bool(&v, 1);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_false(exec_t* ex)
 {
 	var_t v;
 	new_bool(&v, 0);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_null(exec_t* ex)
 {
 	var_t v;
 	new_null(&v);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_symbol(exec_t* ex)
 {
@@ -615,8 +653,43 @@ static int ex_symbol(exec_t* ex)
 	int k;
 	k = read_int(&ex->bc);
 	new_symbol(&v, k);
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
+}
+
+/************************************************************
+ *
+ ************************************************************/
+static int ex_stdin(exec_t* ex)
+{
+	var_t v;
+	new_file(&v, ex->in);
+	push_stack(ex, &v);
+	return -1;
+}
+static int ex_stdout(exec_t* ex)
+{
+	var_t v;
+	new_file(&v, ex->out);
+	push_stack(ex, &v);
+	return -1;
+}
+static int ex_stderr(exec_t* ex)
+{
+	var_t v;
+	new_file(&v, ex->err);
+	push_stack(ex, &v);
+	return -1;
+}
+static int ex_tmpout(exec_t* ex)
+{
+	assert((ex->out = tmpfile()) != NULL);
+	return -1;
+}
+static int ex_tmperr(exec_t* ex)
+{
+	assert((ex->err = tmpfile()) != NULL);
+	return -1;
 }
 
 /************************************************************
@@ -629,14 +702,14 @@ static int ex_array(exec_t* ex)
 	n = read_int(&ex->bc);
 	new_array(ex, &v1, 0);
 	for (i = -n; i < 0; i++) {
-		ex_sp_get(ex, i, &v2);
+		load_stack(ex, i, &v2);
 		array_push(ex, v1.u.p, &v2);
 	}
 	while (n-- > 0) {
-		ex_sp_pop(ex, &v2);
+		pop_stack(ex, &v2);
 	}
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_object(exec_t* ex)
 {
@@ -645,15 +718,15 @@ static int ex_object(exec_t* ex)
 	n = 2 * read_int(&ex->bc);
 	new_object(ex, &v1, 0);
 	for (i = -n; i < 0; i += 2) {
-		ex_sp_get(ex, i+0, &v2);
-		ex_sp_get(ex, i+1, &v3);
-		object_set(ex, v1.u.p, var_toSymbol(&v2), &v3);
+		load_stack(ex, i+0, &v2);
+		load_stack(ex, i+1, &v3);
+		object_set(ex, v1.u.p, var_toSymbol(ex, &v2), &v3);
 	}
 	while (n-- > 0) {
-		ex_sp_pop(ex, &v2);
+		pop_stack(ex, &v2);
 	}
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 
 /************************************************************
@@ -664,32 +737,32 @@ static int ex_match(exec_t* ex)
 	var_t v1,v2;
 	char *s;
 	int n;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	if (var_toString(&v1, &s, &n)) {
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	if (var_toString(ex, &v1, &s, &n)) {
 		n = string_match(ex, s, n, &v2, &v1);
 	}
 	if (!n) {
 		new_bool(&v1, 0);
 	}
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_notmatch(exec_t* ex)
 {
 	var_t v1,v2;
 	char *s;
 	int n;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	if (var_toString(&v1, &s, &n)) {
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	if (var_toString(ex, &v1, &s, &n)) {
 		n = string_match(ex, s, n, &v2, NULL);
 	} else {
 		n = 0;
 	}
 	new_bool(&v1, n == 0);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 
 /************************************************************
@@ -699,126 +772,126 @@ static int ex_typeof(exec_t* ex)
 {
 	var_t v;
 	char *s;
-	ex_sp_pop(ex, &v);
-	s = typeof_var(&v);
+	pop_stack(ex, &v);
+	s = typeof_var(ex, &v);
 	new_string_copy(ex, &v, s, strlen(s));
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 static int ex_toBool(exec_t* ex)
 {
 	var_t v;
 	int r;
-	ex_sp_pop(ex, &v);
-	r = var_toBool(&v);
+	pop_stack(ex, &v);
+	r = var_toBool(ex, &v);
 	new_bool(&v, r);
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 static int ex_toChar(exec_t* ex)
 {
 	var_t v;
 	int r;
-	ex_sp_pop(ex, &v);
-	r = var_toChar(&v);
+	pop_stack(ex, &v);
+	r = var_toChar(ex, &v);
 	new_char(&v, r);
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 static int ex_toInt(exec_t* ex)
 {
 	var_t v;
 	int r;
-	ex_sp_pop(ex, &v);
-	r = var_toInt(&v);
+	pop_stack(ex, &v);
+	r = var_toInt(ex, &v);
 	new_int(&v, r);
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 static int ex_toFloat(exec_t* ex)
 {
 	var_t v;
 	double f;
-	f = var_toFloat(&v);
+	f = var_toFloat(ex, &v);
 	new_float(&v, f);
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 static int ex_toString(exec_t* ex)
 {
 	var_t v;
 	char *s;
 	int n;
-	ex_sp_pop(ex, &v);
-	if (var_toString(&v, &s, &n)) {
+	pop_stack(ex, &v);
+	if (var_toString(ex, &v, &s, &n)) {
 		new_string_copy(ex, &v, s, n);
 	} else {
 		new_null(&v);
 	}
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 static int ex_toNumber(exec_t* ex)
 {
 	var_t v;
 	double f;
 	int i;
-	ex_sp_pop(ex, &v);
-	if (var_toNumber(&v, &f, &i) > 1) {
+	pop_stack(ex, &v);
+	if (var_toNumber(ex, &v, &f, &i) > 1) {
 		new_float(&v, f);
 	} else {
 		new_int(&v, i);
 	}
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 static int ex_plus(exec_t* ex)
 {
 	var_t v;
 	double f;
 	int i;
-	ex_sp_pop(ex, &v);
-	if (var_toNumber(&v, &f, &i) > 1) {
+	pop_stack(ex, &v);
+	if (var_toNumber(ex, &v, &f, &i) > 1) {
 		new_float(&v, f);
 	} else {
 		new_int(&v, i);
 	}
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 static int ex_minus(exec_t* ex)
 {
 	var_t v;
 	double f;
 	int i;
-	ex_sp_pop(ex, &v);
-	if (var_toNumber(&v, &f, &i) > 1) {
+	pop_stack(ex, &v);
+	if (var_toNumber(ex, &v, &f, &i) > 1) {
 		new_float(&v, - f);
 	} else {
 		new_int(&v, - i);
 	}
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 static int ex_not(exec_t* ex)
 {
 	var_t v;
 	int r;
-	ex_sp_pop(ex, &v);
-	r = ! var_toBool(&v);
+	pop_stack(ex, &v);
+	r = ! var_toBool(ex, &v);
 	new_int(&v, r);
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 static int ex_bitnot(exec_t* ex)
 {
 	var_t v;
 	int r;
-	ex_sp_pop(ex, &v);
-	r = ~ var_toInt(&v);
+	pop_stack(ex, &v);
+	r = ~ var_toInt(ex, &v);
 	new_int(&v, r);
-	ex_sp_push(ex, &v); 
-	return 1;
+	push_stack(ex, &v); 
+	return -1;
 }
 
 /************************************************************
@@ -829,28 +902,28 @@ static int ex_inc(exec_t* ex)
 	var_t v;
 	double f;
 	int i;
-	ex_sp_pop(ex, &v);
-	if (var_toNumber(&v, &f, &i) > 1) {
+	pop_stack(ex, &v);
+	if (var_toNumber(ex, &v, &f, &i) > 1) {
 		new_float(&v, f + 1);
 	} else {
 		new_int(&v, i + 1);
 	}
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 static int ex_dec(exec_t* ex)
 {
 	var_t v;
 	double f;
 	int i;
-	ex_sp_pop(ex, &v);
-	if (var_toNumber(&v, &f, &i) > 1) {
+	pop_stack(ex, &v);
+	if (var_toNumber(ex, &v, &f, &i) > 1) {
 		new_float(&v, f - 1);
 	} else {
 		new_int(&v, i - 1);
 	}
-	ex_sp_push(ex, &v);
-	return 1;
+	push_stack(ex, &v);
+	return -1;
 }
 
 /************************************************************
@@ -860,134 +933,134 @@ static int ex_cmp(exec_t* ex)
 {
 	var_t v1,v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_cmp(&v1, &v2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = cmp_vars(ex, &v1, &v2);
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_eq(exec_t* ex)
 {
 	var_t v1,v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_cmp(&v1, &v2) == 0;
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = cmp_vars(ex, &v1, &v2) == 0;
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_ne(exec_t* ex)
 {
 	var_t v1,v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_cmp(&v1, &v2) != 0;
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = cmp_vars(ex, &v1, &v2) != 0;
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_gt(exec_t* ex)
 {
 	var_t v1,v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_cmp(&v1, &v2) > 0;
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = cmp_vars(ex, &v1, &v2) > 0;
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_lt(exec_t* ex)
 {
 	var_t v1,v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_cmp(&v1, &v2) < 0;
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = cmp_vars(ex, &v1, &v2) < 0;
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 
 static int ex_mod(exec_t* ex)
 {
 	var_t v1, v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_toInt(&v1) % var_toInt(&v2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = var_toInt(ex, &v1) % var_toInt(ex, &v2);
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_bitand(exec_t* ex)
 {
 	var_t v1, v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_toInt(&v1) & var_toInt(&v2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = var_toInt(ex, &v1) & var_toInt(ex, &v2);
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_bitor(exec_t* ex)
 {
 	var_t v1, v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_toInt(&v1) | var_toInt(&v2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = var_toInt(ex, &v1) | var_toInt(ex, &v2);
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_bitsub(exec_t* ex)
 {
 	var_t v1, v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_toInt(&v1) & (((unsigned int)-1) ^ var_toInt(&v2));
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = var_toInt(ex, &v1) & (((unsigned int)-1) ^ var_toInt(ex, &v2));
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_bitxor(exec_t* ex)
 {
 	var_t v1, v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_toInt(&v1) ^ var_toInt(&v2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = var_toInt(ex, &v1) ^ var_toInt(ex, &v2);
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_bitshR(exec_t* ex)
 {
 	var_t v1, v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_toInt(&v1) >> var_toInt(&v2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = var_toInt(ex, &v1) >> var_toInt(ex, &v2);
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_bitshL(exec_t* ex)
 {
 	var_t v1, v2;
 	int r;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r = var_toInt(&v1) << var_toInt(&v2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r = var_toInt(ex, &v1) << var_toInt(ex, &v2);
 	new_int(&v1, r);
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 
 static int ex_mul(exec_t* ex)
@@ -995,77 +1068,77 @@ static int ex_mul(exec_t* ex)
 	var_t v1, v2;
 	double f1,f2;
 	int i1,i2,r1,r2;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r1 = var_toNumber(&v1, &f1, &i1);
-	r2 = var_toNumber(&v2, &f2, &i2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r1 = var_toNumber(ex, &v1, &f1, &i1);
+	r2 = var_toNumber(ex, &v2, &f2, &i2);
 	if ((r1 > 1) || (r2 > 1)) {
 		new_float(&v1, f1 * f2);
 	} else {
 		new_int(&v1, i1 * i2);
 	}
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_div(exec_t* ex)
 {
 	var_t v1, v2;
 	double f1,f2;
 	int i1,i2,r1,r2;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r1 = var_toNumber(&v1, &f1, &i1);
-	r2 = var_toNumber(&v2, &f2, &i2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r1 = var_toNumber(ex, &v1, &f1, &i1);
+	r2 = var_toNumber(ex, &v2, &f2, &i2);
 	if ((r1 > 1) || (r2 > 1)) {
 		new_float(&v1, f1 / f2);
 	} else {
 		new_int(&v1, i1 / i2);
 	}
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_pow(exec_t* ex)
 {
 	var_t v1, v2;
 	double f1,f2;
 	int i1,i2,r1,r2;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r1 = var_toNumber(&v1, &f1, &i1);
-	r2 = var_toNumber(&v2, &f2, &i2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r1 = var_toNumber(ex, &v1, &f1, &i1);
+	r2 = var_toNumber(ex, &v2, &f2, &i2);
 	if ((r1 > 1) || (r2 > 1)) {
 		new_float(&v1, pow(f1, f2));
 	} else {
 		new_int(&v1, (int)pow(i1, i2));
 	}
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_sub(exec_t* ex)
 {
 	var_t v1, v2;
 	double f1,f2;
 	int i1,i2,r1,r2;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	r1 = var_toNumber(&v1, &f1, &i1);
-	r2 = var_toNumber(&v2, &f2, &i2);
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	r1 = var_toNumber(ex, &v1, &f1, &i1);
+	r2 = var_toNumber(ex, &v2, &f2, &i2);
 	if ((r1 > 1) || (r2 > 1)) {
 		new_float(&v1, f1 - f2);
 	} else {
 		new_int(&v1, i1 - i2);
 	}
-	ex_sp_push(ex, &v1);
-	return 1;
+	push_stack(ex, &v1);
+	return -1;
 }
 static int ex_add(exec_t* ex)
 {
 	var_t v1, v2, v3;
-	ex_sp_pop(ex, &v2);
-	ex_sp_pop(ex, &v1);
-	var_add(ex, &v1, &v2, &v3);
-	ex_sp_push(ex, &v3);
-	return 1;
+	pop_stack(ex, &v2);
+	pop_stack(ex, &v1);
+	cat_vars(ex, &v1, &v2, &v3);
+	push_stack(ex, &v3);
+	return -1;
 }
 
 /************************************************************
@@ -1092,16 +1165,16 @@ static int ex_main(exec_t* ex)
 		r = 0;
 		if (!(c = *ex->bc++)) {
 			/* EOF */
-			r = 1;
+			r = 0;
 			break;
 		}
 		p = op2ex[c];
 		if (!p) {
-			warns("unknown operand: `%c`\n", c);
+			fprintf(ex->err, "unknown operand: `%c`\n", c);
 		} else {
 			r = (*p)(ex);
 		}
-	} while (r == 1);
+	} while (r == -1);
 	
 	array_free(ex, ex->sp);
 	ex->sp = oldsp;
@@ -1113,16 +1186,29 @@ static int ex_main(exec_t* ex)
 int call_frame(exec_t* ex)
 {
 	object_t* fp;
+	FILE *oldout,*olderr;
 	int r;
 	
+	assert(ex->res->p == NULL);
+	
 	fp = frame_alloc(ex);
 	fp->p = ex->fp;
 	ex->fp = fp;
 	
-	assert(ex->res->type == 0);
+	oldout = ex->out;
+	olderr = ex->err;
 	
 	r = ex_main(ex);
 	
+	if (ex->out != oldout) {
+		fclose(ex->out);
+		ex->out = oldout;
+	}
+	if (ex->err != olderr) {
+		fclose(ex->err);
+		ex->err = olderr;
+	}
+	
 	ex->fp = fp->p;
 	
 	lock_var(ex, ex->res);
@@ -1166,7 +1252,7 @@ int call_function(exec_t* ex, object_t* gp, int sym, char *bc,
 	oldfp = ex->fp;
 	ex->gp = gp;
 	ex->fp = gp;
-	s = symbol_to_chars(sym);
+	s = symbol_to_chars(ex, sym);
 	
 	r = call_bytes(ex, s, bc, argc, argv, res);
 	
@@ -1182,12 +1268,12 @@ int eval_bytes(exec_t* ex, char *sc, int sz, var_t* res)
 	int bn;
 	int r;
 	
-	res->type = 0;
+	res->p = NULL;
 	
 	bc = NULL;
 	bn = 0;
 	
-	r = compile_code_to_bytes(NULL, sc, sz, &bc, &bn);
+	r = compile_code(ex, NULL, sc, sz, &bc, &bn);
 	if (r != 0) {
 		if (bc) free(bc);
 		/* compile error */
@@ -1204,23 +1290,7 @@ int eval_bytes(exec_t* ex, char *sc, int sz, var_t* res)
 /************************************************************
  *
  ************************************************************/
-void ex_trace(exec_t* ex, char *s, ...)
-{
-	va_list va;
-	
-	va_start(va, s);
-	vfprintf(g_stdout, s, va);
-	va_end(va);
-	
-	if (ex->func) {
-		fprintf(g_stdout, " in %s", ex->func);
-	}
-	if (ex->file) {
-		fprintf(g_stdout, " [%s:%d]", ex->file, ex->line);
-	}
-	fputc('\n', g_stdout);
-}
-int exec_bytes(char *bc, int bn)
+int exec_bytes(exec_t* top, char *bc, int bn)
 {
 	exec_t ex;
 	var_t res;
@@ -1231,7 +1301,7 @@ int exec_bytes(char *bc, int bn)
 	/* skip for first void */
 	while (*bc && *bc != 1) bc++;
 	if (!*bc) {
-		/* invalid bytecode */
+		/* bad bytecode */
 		return 0;
 	}
 	
@@ -1241,42 +1311,42 @@ int exec_bytes(char *bc, int bn)
 	fp = frame_alloc(&ex);
 	ex.gp = fp;
 	ex.fp = fp;
+	ex.out = top->out;
+	ex.err = top->err;
+	ex.in  = top->in;
 	
 	r = call_bytes(&ex, "main", bc, 0, NULL, &res);
 	
-	switch (r) {
+	switch (ex.jmp) {
+	case 0:
 	case K_exit:
 	case K_return:
-		if (res.type == V_int) {
+		if (res.p == int_proto) {
 			r = res.u.i;
-		} else if (var_toString(&res, &s, &n)) {
-			fprintf(g_stdout, "%.*s\n", n, s);
-			r = -1;
+		} else if (var_toString(&ex, &res, &s, &n)) {
+			fprintf(ex.out, "%.*s\n", n, s);
+			r = 1;
 		} else {
 			r = 0;
 		}
 		break;
-	case 1:
-		r = 0;
-		break;
-	case K_goto:
-		assert(ex.req != 0);
-		ex_trace(&ex, "invalid label: %s", symbol_to_chars(ex.req));
-		r = -1;
-		break;
-	case -1:
-	case K_throw:
-		if (var_toString(&res, &s, &n)) {
-			ex_trace(&ex, "throwed: %.*s", n, s);
+	case K_catch:
+		if (r) {
+			fprintf(ex.err,
+				"throwed: %s in %s() [%s:%d]\n",
+				var_toChars(&ex, &res), ex.func, ex.file, ex.line);
 		} else {
-			ex_trace(&ex, "uncatched throw");
+			fprintf(ex.err,
+				"error in %s() [%s:%d]\n",
+				ex.func, ex.file, ex.line);
 		}
-		r = -1;
+		r = 1;
 		break;
-	case 0:
 	default:
-		ex_trace(&ex, "invalid result: %d", r);
-		r = -1;
+		fprintf(ex.err,
+			"bad label: %s in %s() [%s:%d]\n",
+			symbol_to_chars(&ex, ex.jmp), ex.func, ex.file, ex.line);
+		r = 1;
 		break;
 	}
 	
@@ -1288,25 +1358,7 @@ int exec_bytes(char *bc, int bn)
 	return r;
 }
 
-int exec_file(char *bcfile)
-{
-	FILE *fp;
-	char *bc;
-	int bn, r;
-	
-	if (!(bc = vmopen(bcfile, &bn, &fp))) {
-		warns("vmopen error: %s %s\n", bcfile, strerror(errno));
-		return -1;
-	}
-	
-	r = exec_bytes(bc, bn);
-	
-	vmclose(bc, bn, fp);
-	
-	return r;
-}
-
-int exec_string(char *sc, int sz)
+int exec_code(exec_t* ex, char *sc, int sz)
 {
 	char *bc;
 	int bn;
@@ -1315,36 +1367,52 @@ int exec_string(char *sc, int sz)
 	bc = NULL;
 	bn = 0;
 	
-	r = compile_code_to_bytes(NULL, sc, sz, &bc, &bn);
+	r = compile_code(ex, NULL, sc, sz, &bc, &bn);
 	if (r != 0) {
 		if (bc) free(bc);
 		/* compile error */
 		return r;
 	}
 	
-	r = exec_bytes(bc, bn);
+	r = exec_bytes(ex, bc, bn);
 	
 	free(bc);
 	
 	return r;
 }
 
-int exec_source(char *scfile)
+int exec_file(exec_t* ex, char *fn)
 {
 	char *bc;
 	int bn, r;
 	
 	bc = NULL;
 	bn = 0;
-	r = compile_file_to_bytes(scfile, &bc, &bn);
+	
+	r = compile_file(ex, fn, &bc, &bn);
 	if (r != 0) {
 		if (bc) free(bc);
 		return r;
 	}
 	
-	r = exec_bytes(bc, bn);
+	r = exec_bytes(ex, bc, bn);
 	
 	free(bc);
 	
 	return r;
 }
+
+int exec_init(exec_t* ex)
+{
+	memset(ex, 0, sizeof(exec_t));
+	ex->out = stdout;
+	ex->err = stderr;
+	ex->in  = stdin;
+	return 0;
+}
+
+int exec_clear(exec_t* ex)
+{
+	return 0;
+}
+
