@@ -6,7 +6,6 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <stdarg.h>
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
@@ -20,25 +19,6 @@
 #include <sys/mman.h>
 #include <sys/time.h>
 
-extern FILE *g_stderr;
-extern FILE *g_stdout;
-
-void trace(char*, ...);
-void warns(char*, ...);
-
-#define warnif(exp, s) { \
-	if (exp) {  \
-		warns("%s: `%s` [%s:%d]\n", s, #exp, __FILE__, __LINE__); \
-		return 0; \
-	}\
-}
-#define errif(exp, s) { \
-	if (exp) {  \
-		warns("%s: `%s` [%s:%d]\n", s, #exp, __FILE__, __LINE__); \
-		return -1; \
-	}\
-}
-
 char* tmpbuf();
 int   sputsn(char*, char*, int);
 int   sputs(char*, char*);
@@ -51,25 +31,15 @@ int   strtoh(char*, int);
 int   strtonum(char*, int, double*, int*);
 int   isoctal(int);
 int   iscrlf(int);
+int   istvfs(int);
 int   xtoi(int);
 int   itoX(int);
 int   ischar(int, char*);
-int   metac(int*, int);
-int   unmetac(char**, int);
+int   c_to_meta(int*, int);
+int   meta_to_c(char**, int);
+char* c_to_entity(int);
+int   entity_to_c(char **);
 int   vmclose(char*, int, FILE*);
 char* vmopen(char*, int*, FILE**);
 
-/************************************************************
- *
- ************************************************************/
-typedef struct {
-	FILE* fp;
-	char *sp;
-	int len;
-} stream_t;
-
-int stream_putc(stream_t* st, int c);
-int stream_write(stream_t* st, char *s, int n);
-int stream_flush(stream_t* st);
-
 #endif /* _UTIL_H_ */
