@@ -9,10 +9,9 @@
 /************************************************************
  *
  ************************************************************/
-name_t* g_names = NULL;
-
-int string_to_symbol(char *s, int n)
+int string_to_symbol(exec_t* ex, char *s, int n)
 {
+	name_t* ns;
 	int k,i;
 	k = strtoh(s, n);
 	if (!k) return 0;
@@ -20,36 +19,36 @@ int string_to_symbol(char *s, int n)
 #include "_sym2sym.h"	
 	}
 	i = 0;
-	if (g_names) {
-		for (; g_names[i].s; i++) {
-			if (g_names[i].k == k) {
+	if ((ns = ex->ns) != NULL) {
+		for (; ns->s; ns++, i++) {
+			if (ns->k == k) {
 				return k;
 			}
 		}
 	}
-	g_names = realloc(g_names, (i + 2) * sizeof(name_t));
-	g_names[i].k = k;
-	g_names[i].s = strndupok(s, n);
-	g_names[i].n = n;
-	memset(g_names + i + 1, 0, sizeof(name_t));
+	ex->ns = realloc(ex->ns, (i + 2) * sizeof(name_t));
+	ns = ex->ns + i;
+	ns->k = k;
+	ns->s = strndupok(s, n);
+	ns->n = n;
+	memset(ns + 1, 0, sizeof(name_t));
 	return k;
 }
-int chars_to_symbol(char* s)
+int chars_to_symbol(exec_t* ex, char* s)
 {
-	return string_to_symbol(s, strlen(s));
+	return string_to_symbol(ex, s, strlen(s));
 }
-int symbol_to_string(int k, char** s, int *n)
+int symbol_to_string(exec_t* ex, int k, char** s, int *n)
 {
-	int i;
+	name_t* ns;
 	switch (k) {
 #include "_sym2str.h"	
 	}
-	i = 0;
-	if (g_names) {
-		for (; g_names[i].s; i++) {
-			if (g_names[i].k == k) {
-				*s = g_names[i].s;
-				*n = g_names[i].n;
+	if ((ns = ex->ns) != NULL) {
+		for (; ns->s; ns++) {
+			if (ns->k == k) {
+				*s = ns->s;
+				*n = ns->n;
 				return *n;
 			}
 		}
@@ -58,107 +57,110 @@ int symbol_to_string(int k, char** s, int *n)
 	*n = 0;
 	return 0;
 }
-char* symbol_to_chars(int k)
+char* symbol_to_chars(exec_t* ex, int k)
 {
 	char *s;
 	int n;
-	if (symbol_to_string(k, &s, &n)) {
+	if (symbol_to_string(ex, k, &s, &n)) {
 		return s;
 	}
 	return NULL;
 }
-void free_names()
+void free_names(exec_t* ex)
 {
-	int i = 0;
-	if (g_names) {
-		for (; g_names[i].s; i++) {
-			free(g_names[i].s);
+	name_t* ns;
+	if ((ns = ex->ns) != NULL) {
+		for (; ns->s; ns++) {
+			free(ns->s);
 		}
-		free(g_names);
-		g_names = NULL;
+		free(ex->ns);
+		ex->ns = NULL;
 	}
 }
 
 /************************************************************
  *
  ************************************************************/
-void* alloc_link(exec_t* ex, size_t sz, proto_t* proto)
-{
-	link_t* p;
-	p = calloc(1, sizeof(link_t) + sz);
-	p->proto = proto;
-	p->prev = ex->gc;
-	if (ex->gc) {
-		ex->gc->next = p;
-	}
-	ex->gc = p;
-	return ((void*)p + sizeof(link_t));
-}
-void trace_link(link_t* p)
-{
-	void *v = (void*)p + sizeof(link_t);
-	trace("%p: %d: %s\n", p, p->lock, p->proto->typestr);
-	if (p->proto->trace) {
-		(*p->proto->trace)(v);
-	}
-}
 void trace_links(exec_t* ex)
 {
-	link_t* p = ex->gc;
-	while (p) {
-		trace_link(p);
-		p = p->prev;
+	link_t* l = ex->gc;
+	while (l) {
+		fprintf(ex->out, "%p: %d: %s\n", l, l->lock, l->proto->typestr);
+		if (l->proto->trace) {
+			(*l->proto->trace)(ex, l + 1);
+		}
+		l = l->prev;
 	}
 }
-void* free_link(exec_t* ex, void *v)
+void* alloc_link(exec_t* ex, size_t sz, proto_t* proto)
 {
-	link_t *p, *f;
-	p = v - sizeof(link_t);
-	if (p->proto && p->proto->clear) {
-		(*p->proto->clear)(ex, v);
+	link_t *l;
+	l = calloc(1, sizeof(link_t) + sz);
+	l->proto = proto;
+	l->prev = ex->gc;
+	if (ex->gc) {
+		ex->gc->next = l;
 	}
-	if ((f = p->prev) != NULL) {
-		f->next = p->next;
+	ex->gc = l;
+	return (void*)(l + 1);
+}
+static link_t* remove_link(exec_t* ex, link_t* l)
+{
+	link_t *prev, *next;
+	if (l->proto && l->proto->clear) {
+		(*l->proto->clear)(ex, l + 1);
+		/* l->prev && l->next 
+		can be modified
+		 when clear object or array.
+		 */
 	}
-	if (p->next) {
-		p->next->prev = f;
+	prev = l->prev;
+	next = l->next;
+	free(l);
+	if (prev) {
+		prev->next = next;
+	}
+	if (next) {
+		next->prev = prev;
 	} else {
-		ex->gc = f;
+		ex->gc = prev;
 	}
-	free(p);
-	return f;
+	return prev;
 }
-void free_links(exec_t* ex, void *v)
+void free_link(exec_t* ex, void *p)
 {
-	link_t *p;
-	p = v - sizeof(link_t);
-	while (p->next) {
-		if (p->next->lock > 0) {
-			p = p->next;
+	remove_link(ex, (link_t*)p - 1);
+}
+void free_links(exec_t* ex, void *p)
+{
+	link_t *l = (link_t*)p - 1;
+	while (l->next) {
+		if (l->next->lock > 0) {
+			l = l->next;
 		} else {
-			p = free_link(ex, (void*)p->next + sizeof(link_t));
+			l = remove_link(ex, l->next);
 		}
 	}
-	assert(p == ex->gc);
-	free_link(ex, v);
+	assert(l == ex->gc);
+	free_link(ex, p);
 	if (!ex->gc) {
-		free_names();
+		free_names(ex);
 	}
 }
 
 /************************************************************
  *
  ************************************************************/
-void trace_var(var_t* v)
+void trace_var(exec_t* ex, var_t* v)
 {
-	trace("%s: %s\n", all_protos[v->type]->typestr, var_toChars(v));
-	if (all_protos[v->type]->trace) {
-		(*all_protos[v->type]->trace)(v->u.p);
+	fprintf(ex->out, "%s: %s\n", typeof_var(ex, v), var_toChars(ex, v));
+	if (v->p && v->p->trace) {
+		(*v->p->trace)(ex, v->u.p);
 	}
 }
 int lock_var(exec_t* ex, var_t* v)
 {
-	if (all_protos[v->type]->lock) {
+	if (v->p && v->p->lock) {
 		((link_t*)(v->u.p - sizeof(link_t)))->lock++;
 		return 1;
 	}
@@ -166,7 +168,7 @@ int lock_var(exec_t* ex, var_t* v)
 }
 int unlock_var(exec_t* ex, var_t* v)
 {
-	if (all_protos[v->type]->lock) {
+	if (v->p && v->p->lock) {
 		((link_t*)(v->u.p - sizeof(link_t)))->lock--;
 		return 1;
 	}
@@ -174,151 +176,168 @@ int unlock_var(exec_t* ex, var_t* v)
 }
 int delete_var(exec_t* ex, var_t* v)
 {
-	if (all_protos[v->type]->lock) {
+	if (v->p && v->p->lock) {
 		if (((link_t*)(v->u.p - sizeof(link_t)))->lock <= 0) {
 			free_link(ex, v->u.p);
 			return 1;
 		}
 	}
+	/* else {
+		if (v->p && v->p->clear) {
+			(*v->p->clear)(ex, v->u.p);
+			return 1;
+		}
+	}
+	*/
 	return 0;
 }
+
+/************************************************************
+ * MEMCPY
+ ************************************************************/
 void copy_var(var_t* v1, var_t* v2)
 {
 	memcpy(v1, v2, sizeof(var_t));
 }
-char* typeof_var(var_t* v)
+
+/************************************************************
+ * GET TYPESTR
+ ************************************************************/
+char* typeof_var(exec_t* ex, var_t* v)
 {
-	return all_protos[v->type]->typestr;
+	return (v->p ? v->p->typestr : "null");
 }
 
 /************************************************************
- *
+ * CONCAT
  ************************************************************/
-int var_add(exec_t* ex, var_t* v1, var_t* v2, var_t* v3)
+int cat_vars(exec_t* ex, var_t* v1, var_t* v2, var_t* v3)
 {
-	if (v1->type == V_string || v2->type == V_string) {
+	if (v1->p == string_proto || v2->p == string_proto) {
 		char *s1, *s2;
 		int n1, n2;
 		new_string(ex, v3, NULL);
-		if (var_toString(v1, &s1, &n1)) {
+		if (var_toString(ex, v1, &s1, &n1)) {
 			string_add(v3->u.p, s1, n1);
 		}
-		if (var_toString(v2, &s2, &n2)) {
+		if (var_toString(ex, v2, &s2, &n2)) {
 			string_add(v3->u.p, s2, n2);
 		}
 	} else 
-	if (v1->type == V_float || v2->type == V_float) {
-		double r = var_toFloat(v1) + var_toFloat(v2);
+	if (v1->p == float_proto || v2->p == float_proto) {
+		double r = var_toFloat(ex, v1) + var_toFloat(ex, v2);
 		new_float(v3, r);
 	} else {
-		int r = var_toInt(v1) + var_toInt(v2);
+		int r = var_toInt(ex, v1) + var_toInt(ex, v2);
 		new_int(v3, r);
 	}
 	return 1;
 }
 
 /************************************************************
- *
+ * COMPARE
  ************************************************************/
-int var_cmpAsNumber(var_t* v1, var_t* v2)
+int cmp_vars_asNumber(exec_t* ex, var_t* v1, var_t* v2)
 {
 	double f1, f2;
-	f1 = var_toFloat(v1);
-	f2 = var_toFloat(v2);
+	f1 = var_toFloat(ex, v1);
+	f2 = var_toFloat(ex, v2);
 	return ((f1 < f2) ? -1 : (f1 > f2));
 }
-int var_cmpAsString(var_t* v1, var_t* v2)
+int cmp_vars_asString(exec_t* ex, var_t* v1, var_t* v2)
 {
 	char *s1,*s2;
 	int n1, n2;
 	if (v1->u.p == v2->u.p) return 0;
-	if (!var_toString(v1, &s1, &n1)
-	|| !var_toString(v2, &s2, &n2)) {
+	if (!var_toString(ex, v1, &s1, &n1)
+	|| !var_toString(ex, v2, &s2, &n2)) {
 		return (n1 < n2) ? -1 : (n1 > n2);
 	}
 	return strnncmp(s1, n1, s2, n2);
 }
-int var_cmp(var_t* v1, var_t* v2)
+int cmp_vars(exec_t* ex, var_t* v1, var_t* v2)
 {
-	if (v1->type == V_string || v2->type == V_string) {
-		return var_cmpAsString(v1, v2);
-	} else if (v1->type == V_float || v2->type == V_float) {
-		return var_cmpAsNumber(v1, v2);
+	if (v1->p == string_proto || v2->p == string_proto) {
+		return cmp_vars_asString(ex, v1, v2);
+	} else if (v1->p == float_proto || v2->p == float_proto) {
+		return cmp_vars_asNumber(ex, v1, v2);
 	}		
-	return (var_toInt(v1) - var_toInt(v2));
+	return (var_toInt(ex, v1) - var_toInt(ex, v2));
 }
 
 /************************************************************
  *
  ************************************************************/
-int var_toBool(var_t* v)
+int var_toBool(exec_t* ex, var_t* v)
 {
-	if (all_protos[v->type]->toBool) {
-		return (*all_protos[v->type]->toBool)(v);
+	if (v->p && v->p->toBool) {
+		return (*v->p->toBool)(ex, v);
 	}
-	return (v->type != 0);
+	return (v->p != NULL);
 }
-int var_toChar(var_t* v)
+int var_toChar(exec_t* ex, var_t* v)
 {
-	if (all_protos[v->type]->toChar) {
-		return (*all_protos[v->type]->toChar)(v);
+	if (v->p && v->p->toChar) {
+		return (*v->p->toChar)(ex, v);
 	}
 	return 0;
 }
-int var_toInt(var_t* v)
+int var_toInt(exec_t* ex, var_t* v)
 {
-	if (all_protos[v->type]->toInt) {
-		return (*all_protos[v->type]->toInt)(v);
+	if (v->p && v->p->toInt) {
+		return (*v->p->toInt)(ex, v);
 	}
 	return 0;
 }
-double var_toFloat(var_t* v)
+double var_toFloat(exec_t* ex, var_t* v)
 {
-	if (all_protos[v->type]->toFloat) {
-		return (*all_protos[v->type]->toFloat)(v);
+	if (v->p && v->p->toFloat) {
+		return (*v->p->toFloat)(ex, v);
 	}
 	return 0;
 }
-int var_toString(var_t* v, char **s, int *n)
+int var_toString(exec_t* ex, var_t* v, char **s, int *n)
 {
-	if (all_protos[v->type]->toString) {
-		return (*all_protos[v->type]->toString)(v, s, n);
+	if (v->p && v->p->toString) {
+		return (*v->p->toString)(ex, v, s, n);
 	}
 	*s = NULL;
 	*n = 0;
 	return 0;
 }
-int var_toNumber(var_t* v, double *f, int *i)
+int var_toNumber(exec_t* ex, var_t* v, double *f, int *i)
 {
-	if (v->type == V_float) {
-		*i = *f = v->u.f;
-		return 2;
-	} else if (all_protos[v->type]->toNumber) {
-		return (*all_protos[v->type]->toNumber)(v, f, i);
-	} else if (all_protos[v->type]->toInt) {
-		*f = *i = (*all_protos[v->type]->toInt)(v);
-		return 1;
+	if (v->p) {
+		if (v->p == float_proto) {
+			*i = *f = v->u.f;
+			return 2;
+		} else if (v->p->toNumber) {
+			return (*v->p->toNumber)(ex, v, f, i);
+		} else if (v->p->toInt) {
+			*f = *i = (*v->p->toInt)(ex, v);
+			return 1;
+		}
 	}
 	*f = *i = 0;
 	return 0;
 }
-char* var_toChars(var_t* v)
+char* var_toChars(exec_t* ex,var_t* v)
 {
 	char *s;
 	int n;
-	if (var_toString(v, &s, &n)) {
+	if (var_toString(ex, v, &s, &n)) {
 		return s;
 	}
 	return NULL;
 }
-int var_toSymbol(var_t* v)
+int var_toSymbol(exec_t* ex, var_t* v)
 {
 	char *s;
 	int n;
-	if (v->type == V_symbol) {
+	if (v->p == symbol_proto) {
 		return v->u.i;
-	} else if (var_toString(v, &s, &n)) {
-		return string_to_symbol(s, n);
+	} else if (var_toString(ex, v, &s, &n)) {
+		return string_to_symbol(ex, s, n);
 	}
 	return 0;
 }
@@ -328,60 +347,63 @@ int var_toSymbol(var_t* v)
  ************************************************************/
 int var_getLength(exec_t* ex, var_t* v, var_t* res)
 {
-	if (all_protos[v->type]->getLength) {
-		new_int(res, (*all_protos[v->type]->getLength)(ex, v->u.p));
+	if (v->p && v->p->getLength) {
+		new_int(res, (*v->p->getLength)(ex, v->u.p));
 		return 1;
 	}
 	new_null(res);
-	warns("invalid getLength: %s\n", typeof_var(v));
+	fprintf(ex->err, "bad getLength: %s\n", typeof_var(ex, v));
 	return 0;
 }
 int var_setLength(exec_t* ex, var_t* v, var_t* val)
 {
-	warnif (val->type != V_int, "invalid value");
-	if (all_protos[v->type]->setLength) {
-		return (*all_protos[v->type]->setLength)(ex, v->u.p, val->u.i);
+	if (val->p != int_proto) {
+		fprintf(ex->err, "bad value: %s\n", typeof_var(ex, val));
+		return 0;
 	}
-	warns("invalid setLength: %s\n", typeof_var(v));
+	if (v->p && v->p->setLength) {
+		return (*v->p->setLength)(ex, v->u.p, val->u.i);
+	}
+	fprintf(ex->err, "bad setLength: %s\n", typeof_var(ex, v));
 	return 0;
 }
 int var_getItem(exec_t* ex, var_t* v, var_t* key, var_t* res)
 {
-	if ((key->type == V_symbol) && (key->u.i == K_length)) {
+	if ((key->p == symbol_proto) && (key->u.i == K_length)) {
 		return var_getLength(ex, v, res);
 	}
-	if (all_protos[v->type]->getItem) {
-		return (*all_protos[v->type]->getItem)(ex, v->u.p, key, res);
+	if (v->p && v->p->getItem) {
+		return (*v->p->getItem)(ex, v->u.p, key, res);
 	}
 	new_null(res);
 	return 0;
 }
 int var_setItem(exec_t* ex, var_t* v, var_t* key, var_t* val)
 {
-	if ((key->type == V_symbol) && (key->u.i == K_length)) {
+	if ((key->p == symbol_proto) && (key->u.i == K_length)) {
 		return var_setLength(ex, v, val);
 	}
-	if (all_protos[v->type]->setItem) {
-		return (*all_protos[v->type]->setItem)(ex, v->u.p, key, val);
+	if (v->p && v->p->setItem) {
+		return (*v->p->setItem)(ex, v->u.p, key, val);
 	}
-	warns("invalid setItem: %s\n", typeof_var(v));
+	fprintf(ex->err, "bad setItem: %s\n", typeof_var(ex, v));
 	return 0;
 }
 int var_delItem(exec_t* ex, var_t* v, var_t* key, var_t* res)
 {
-	if (all_protos[v->type]->delItem) {
-		return (*all_protos[v->type]->delItem)(ex, v->u.p, key, res);
+	if (v->p && v->p->delItem) {
+		return (*v->p->delItem)(ex, v->u.p, key, res);
 	}
 	if (res) new_null(res);
-	warns("invalid delItem: %s\n", typeof_var(v));
+	fprintf(ex->err, "bad delItem: %s\n", typeof_var(ex, v));
 	return 0;
 }
 int var_eachItem(exec_t* ex, var_t* v, int i, var_t* key, var_t* val)
 {
-	if (all_protos[v->type]->eachItem) {
-		return (*all_protos[v->type]->eachItem)(ex, v->u.p, i, key, val);
+	if (v->p && v->p->eachItem) {
+		return (*v->p->eachItem)(ex, v->u.p, i, key, val);
 	}
-	warns("invalid eachItem: %s\n", typeof_var(v));
+	fprintf(ex->err, "bad eachItem: %s\n", typeof_var(ex, v));
 	return 0;
 }
 int var_callFunc(exec_t* ex, var_t* v, int sym, int argc, var_t* argv, var_t* res)
@@ -390,19 +412,21 @@ int var_callFunc(exec_t* ex, var_t* v, int sym, int argc, var_t* argv, var_t* re
 	if (!v) {
 		r = global_callFunc(ex, sym, argc, argv, res);
 		if (r >= 0) return r;
-		warns("no such function: %s(%d)\n", symbol_to_chars(sym), argc);
+		fprintf(ex->err, "no such function: %s(%d)\n", symbol_to_chars(ex, sym), argc);
 	} else {
-		if (all_protos[v->type]->callFunc) {
-			r = (*all_protos[v->type]->callFunc)(ex, v->u.p, sym, argc, argv, res);
+		if (v->p && v->p->callFunc) {
+			r = (*v->p->callFunc)(ex, v->u.p, sym, argc, argv, res);
 			if (r >= 0) return r;
 		}
 		if (argc == 0) {
 			r = global_callFunc(ex, sym, 1, v, res);
 			if (r >= 0) return r;
 		}
-		warns("no such function: %s.%s(%d)\n", typeof_var(v), symbol_to_chars(sym), argc);
+		fprintf(ex->err, "no such function: %s.%s(%d)\n",
+			typeof_var(ex, v), symbol_to_chars(ex, sym), argc);
 	}
-	return -1;
+	ex->jmp = K_catch;
+	return 0;
 }
 
 /************************************************************
@@ -410,62 +434,57 @@ int var_callFunc(exec_t* ex, var_t* v, int sym, int argc, var_t* argv, var_t* re
  ************************************************************/
 void new_null (var_t* v)
 {
-	v->type = 0;
+	v->p = NULL;
 }
 void new_bool(var_t* v, int i)
 {
-	v->type = V_bool;
+	v->p = bool_proto;
 	v->u.i = i != 0;
 }
 void new_char (var_t* v, int i)
 {
-	v->type = V_char;
+	v->p = char_proto;
 	v->u.i = i & 0xFF;
 }
 void new_int (var_t* v, int i)
 {
-	v->type = V_int;
+	v->p = int_proto;
 	v->u.i = i;
 }
 void new_symbol (var_t* v, int i)
 {
-	v->type = V_symbol;
+	v->p = symbol_proto;
 	v->u.i = i;
 }
 void new_float (var_t* v, double f)
 {
-	v->type = V_float;
+	v->p = float_proto;
 	v->u.f = f;
 }
 void new_label (var_t* v, char *p)
 {
-	v->type = V_label;
+	v->p = label_proto;
 	v->u.p = p;
 }
 void new_function (var_t* v, char *p)
 {
-	v->type = V_function;
+	v->p = function_proto;
 	v->u.p = p;
 }
 void new_class (var_t* v, char *p)
 {
-	v->type = V_class;
+	v->p = class_proto;
 	v->u.p = p;
 }
 void new_file(var_t* v, FILE* fp)
 {
-	v->type = V_file;
+	v->p = file_proto;
 	v->u.p = fp;
 }
-void new_dbm(var_t* v, void *p)
-{
-	v->type = V_dbm;
-	v->u.p = p;
-}
 void new_string_with (exec_t* ex, var_t* v, char *s, int n)
 {
 	string_t* str;
-	v->type = V_string;
+	v->p = string_proto;
 	str = v->u.p = string_alloc(ex);
 	if (s && n) {
 		str->s = s;
@@ -475,7 +494,7 @@ void new_string_with (exec_t* ex, var_t* v, char *s, int n)
 void new_string_copy (exec_t* ex, var_t* v, char *s, int n)
 {
 	string_t* str;
-	v->type = V_string;
+	v->p = string_proto;
 	str = v->u.p = string_alloc(ex);
 	if (s && n) {
 		str->s = strndupok(s, n);
@@ -485,60 +504,63 @@ void new_string_copy (exec_t* ex, var_t* v, char *s, int n)
 void new_string (exec_t* ex, var_t* v, void *p)
 {
 	if (!p) p = string_alloc(ex);
-	v->type = V_string;
+	v->p = string_proto;
 	v->u.p = p;
 }
 void new_regex (exec_t* ex, var_t* v, char *s, int n, int i)
 {
-	v->type = V_regex;
+	v->p = regex_proto;
 	v->u.p = regex_alloc(ex);
-	if (!regex_comp(v->u.p, s, n, i)) {
-		/* make null */;
+	if (!regex_comp(ex, v->u.p, s, n, i)) {
+		fprintf(ex->err, "regex compile error: %.*s\n", n, s);
 		regex_free(ex, v->u.p);
-		v->type = 0;
+		/* make null */;
+		v->p = NULL;
 	}
 }
 void new_array (exec_t* ex, var_t* v, void *p)
 {
 	if (!p) p = array_alloc(ex);
-	v->type = V_array;
+	v->p = array_proto;
 	v->u.p = p;
 }
 void new_refer (exec_t* ex, var_t* v, void *p)
 {
 	if (!p) p = refer_alloc(ex);
-	v->type = V_refer;
+	v->p = refer_proto;
 	v->u.p = p;
 }
 void new_object (exec_t* ex, var_t* v, void* p)
 {
 	if (!p) p = object_alloc(ex);
-	v->type = V_object;
+	v->p = object_proto;
 	v->u.p = p;
 }
 void new_frame (exec_t* ex, var_t* v, void *p)
 {
-	if (!p) p = frame_alloc(ex);
-	v->type = V_frame;
+	if (!p) {
+		p = frame_alloc(ex);
+	}
+	v->p = frame_proto;
 	v->u.p = p;
 }
 
 /************************************************************
  *
  ************************************************************/
-static int int_toBool(var_t* v)
+static int int_toBool(exec_t* ex, var_t* v)
 {
 	return (v->u.i != 0);
 }
-static int int_toChar(var_t* v)
+static int int_toChar(exec_t* ex, var_t* v)
 {
 	return (v->u.i & 0xFF);
 }
-static int int_toInt(var_t* v)
+static int int_toInt(exec_t* ex, var_t* v)
 {
 	return v->u.i;
 }
-static double int_toFloat(var_t* v)
+static double int_toFloat(exec_t* ex, var_t* v)
 {
 	return (double)v->u.i;
 }
@@ -546,19 +568,19 @@ static double int_toFloat(var_t* v)
 /************************************************************
  *
  ************************************************************/
-static int float_toBool(var_t* v)
+static int float_toBool(exec_t* ex, var_t* v)
 {
 	return (v->u.f != 0);
 }
-static int float_toChar(var_t* v)
+static int float_toChar(exec_t* ex, var_t* v)
 {
 	return ((int)v->u.f & 0xFF);
 }
-static int float_toInt(var_t* v)
+static int float_toInt(exec_t* ex, var_t* v)
 {
 	return (int)v->u.f;
 }
-static double float_toFloat(var_t* v)
+static double float_toFloat(exec_t* ex, var_t* v)
 {
 	return v->u.f;
 }
@@ -566,7 +588,7 @@ static double float_toFloat(var_t* v)
 /************************************************************
  *
  ************************************************************/
-static int bool_toString(var_t* v, char **s, int *n)
+static int bool_toString(exec_t* ex, var_t* v, char **s, int *n)
 {
 	if (v->u.i) {
 		*s = "true";
@@ -577,48 +599,27 @@ static int bool_toString(var_t* v, char **s, int *n)
 	}
 	return *n;
 }
-static int char_toString(var_t* v, char **s, int *n)
+static int char_toString(exec_t* ex, var_t* v, char **s, int *n)
 {
 	return (*n = sprintf(*s = tmpbuf(), "%c", (v->u.i & 0xFF)));
 }
-static int int_toString(var_t* v, char **s, int *n)
+static int int_toString(exec_t* ex, var_t* v, char **s, int *n)
 {
 	return (*n = sprintf(*s = tmpbuf(), "%d", v->u.i));
 }
-static int float_toString(var_t* v, char **s, int *n)
+static int float_toString(exec_t* ex, var_t* v, char **s, int *n)
 {
 	return (*n = sprintf(*s = tmpbuf(), "%g", v->u.f));
 }
-static int symbol_toString(var_t* v, char **s, int *n)
+static int symbol_toString(exec_t* ex, var_t* v, char **s, int *n)
 {
-	return symbol_to_string(v->u.i, s, n);
+	return symbol_to_string(ex, v->u.i, s, n);
 }
 
 /************************************************************
  *
  ************************************************************/
-proto_t null_proto = {
-"null", /* typestr */
-0, /* lock */
-NULL, /* trace */
-NULL, /* clear */
-NULL, /* toBool */
-NULL, /* toChar */
-NULL, /* toInt */
-NULL, /* toFloat */
-NULL, /* toString */
-NULL, /* toNumber */
-NULL, /* getLength */
-NULL, /* setLegnth */
-NULL, /* callFunc */
-NULL, /* getItem */
-NULL, /* setItem */
-NULL, /* delItem */
-NULL, /* eachItem */
-0
-};
-
-proto_t bool_proto = {
+proto_t _bool_proto_ = {
 "bool", /* typestr */
 0, /* lock */
 NULL, /* trace */
@@ -627,8 +628,8 @@ int_toBool, /* toBool */
 int_toChar, /* toChar */
 int_toInt, /* toInt */
 int_toFloat, /* toFloat */
-bool_toString, /* toString */
 NULL, /* toNumber */
+bool_toString, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -639,7 +640,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t char_proto = {
+proto_t _char_proto_ = {
 "char", /* typestr */
 0, /* lock */
 NULL, /* trace */
@@ -648,8 +649,8 @@ int_toBool, /* toBool */
 int_toChar, /* toChar */
 int_toInt, /* toInt */
 int_toFloat, /* toFloat */
-char_toString, /* toString */
 NULL, /* toNumber */
+char_toString, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -660,7 +661,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t int_proto = {
+proto_t _int_proto_ = {
 "int", /* typestr */
 0, /* lock */
 NULL, /* trace */
@@ -669,8 +670,8 @@ int_toBool, /* toBool */
 int_toChar, /* toChar */
 int_toInt, /* toInt */
 int_toFloat, /* toFloat */
-int_toString, /* toString */
 NULL, /* toNumber */
+int_toString, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -681,7 +682,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t symbol_proto = {
+proto_t _symbol_proto_ = {
 "symbol", /* typestr */
 0, /* lock */
 NULL, /* trace */
@@ -690,8 +691,8 @@ NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
 NULL, /* toFloat */
-symbol_toString, /* toString */
 NULL, /* toNumber */
+symbol_toString, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -702,7 +703,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t float_proto = {
+proto_t _float_proto_ = {
 "float", /* typestr */
 0, /* lock */
 NULL, /* trace */
@@ -711,8 +712,8 @@ float_toBool, /* toBool */
 float_toChar, /* toChar */
 float_toInt, /* toInt */
 float_toFloat, /* toFloat */
-float_toString, /* toString */
 NULL, /* toNumber */
+float_toString, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -723,7 +724,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t string_proto = {
+proto_t _string_proto_ = {
 "string", /* typestr */
 1, /* lock */
 string_trace, /* trace */
@@ -732,8 +733,8 @@ string_toBool, /* toBool */
 string_toChar, /* toChar */
 string_toInt, /* toInt */
 string_toFloat, /* toFloat */
-string_toString, /* toString */
 string_toNumber, /* toNumber */
+string_toString, /* toString */
 string_getLength, /* getLength */
 string_setLength, /* setLegnth */
 string_callFunc, /* callFunc */
@@ -744,7 +745,7 @@ string_eachItem, /* eachItem */
 0
 };
 
-proto_t regex_proto = {
+proto_t _regex_proto_ = {
 "regex", /* typestr */
 1, /* lock */
 regex_trace, /* trace */
@@ -753,8 +754,8 @@ NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
 NULL, /* toFloat */
-NULL, /* toString */
 NULL, /* toNumber */
+NULL, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -765,7 +766,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t array_proto = {
+proto_t _array_proto_ = {
 "array", /* typestr */
 1, /* lock */
 array_trace, /* trace */
@@ -774,8 +775,8 @@ array_toBool, /* toBool */
 array_toChar, /* toChar */
 array_toInt, /* toInt */
 array_toFloat, /* toFloat */
-array_toString, /* toString */
 NULL, /* toNumber */
+array_toString, /* toString */
 array_getLength, /* getLength */
 array_setLength, /* setLegnth */
 array_callFunc, /* callFunc */
@@ -786,7 +787,7 @@ array_eachItem, /* eachItem */
 0
 };
 
-proto_t refer_proto = {
+proto_t _refer_proto_ = {
 "refer", /* typestr */
 1, /* lock */
 refer_trace, /* trace */
@@ -795,8 +796,8 @@ NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
 NULL, /* toFloat */
-NULL, /* toString */
 NULL, /* toNumber */
+NULL, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -807,7 +808,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t object_proto = {
+proto_t _object_proto_ = {
 "object", /* typestr */
 1, /* lock */
 object_trace, /* trace */
@@ -816,8 +817,8 @@ object_toBool, /* toBool */
 object_toChar, /* toChar */
 object_toInt, /* toInt */
 object_toFloat, /* toFloat */
-object_toString, /* toString */
 NULL, /* toNumber */
+object_toString, /* toString */
 object_getLength, /* getLength */
 object_setLength, /* setLegnth */
 object_callFunc, /* callFunc */
@@ -828,7 +829,7 @@ object_eachItem, /* eachItem */
 0
 };
 
-proto_t frame_proto = {
+proto_t _frame_proto_ = {
 "frame", /* typestr */
 0, /* lock */
 frame_trace, /* trace */
@@ -837,8 +838,8 @@ NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
 NULL, /* toFloat */
-NULL, /* toString */
 NULL, /* toNumber */
+NULL, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -849,7 +850,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t file_proto = {
+proto_t _file_proto_ = {
 "file", /* typestr */
 0, /* lock */
 NULL, /* trace */
@@ -858,8 +859,8 @@ NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
 NULL, /* toFloat */
-NULL, /* toString */
 NULL, /* toNumber */
+NULL, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 file_callFunc, /* callFunc */
@@ -870,7 +871,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t dbm_proto = {
+proto_t _dbm_proto_ = {
 "dbm", /* typestr */
 0, /* lock */
 NULL, /* trace */
@@ -879,8 +880,8 @@ NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
 NULL, /* toFloat */
-NULL, /* toString */
 NULL, /* toNumber */
+NULL, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 dbm_callFunc, /* callFunc */
@@ -891,7 +892,28 @@ dbm_eachItem, /* eachItem */
 0
 };
 
-proto_t function_proto = {
+proto_t _xml_proto_ = {
+"xml", /* typestr */
+1, /* lock */
+xml_trace, /* trace */
+xml_clear, /* clear */
+NULL, /* toBool */
+NULL, /* toChar */
+NULL, /* toInt */
+NULL, /* toFloat */
+NULL, /* toNumber */
+xml_toString, /* toString */
+xml_getLength, /* getLength */
+NULL, /* setLegnth */
+xml_callFunc, /* callFunc */
+NULL, /* getItem */
+NULL, /* setItem */
+NULL, /* delItem */
+NULL, /* eachItem */
+0
+};
+
+proto_t _function_proto_ = {
 "function", /* typestr */
 0, /* lock */
 NULL, /* trace */
@@ -900,8 +922,8 @@ NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
 NULL, /* toFloat */
-NULL, /* toString */
 NULL, /* toNumber */
+NULL, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -912,7 +934,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t class_proto = {
+proto_t _class_proto_ = {
 "class", /* typestr */
 0, /* lock */
 NULL, /* trace */
@@ -921,8 +943,8 @@ NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
 NULL, /* toFloat */
-NULL, /* toString */
 NULL, /* toNumber */
+NULL, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -933,7 +955,7 @@ NULL, /* eachItem */
 0
 };
 
-proto_t label_proto = {
+proto_t _label_proto_ = {
 "label", /* typestr */
 0, /* lock */
 NULL, /* trace */
@@ -942,8 +964,8 @@ NULL, /* toBool */
 NULL, /* toChar */
 NULL, /* toInt */
 NULL, /* toFloat */
-NULL, /* toString */
 NULL, /* toNumber */
+NULL, /* toString */
 NULL, /* getLength */
 NULL, /* setLegnth */
 NULL, /* callFunc */
@@ -954,24 +976,20 @@ NULL, /* eachItem */
 0
 };
 
-proto_t* all_protos[] = {
-	& null_proto,
-	& bool_proto,
-	& char_proto,
-	& int_proto,
-	& symbol_proto,
-	& float_proto,
-	& file_proto,
-	& dbm_proto,
-	& function_proto,
-	& class_proto,
-	& label_proto,
-	& frame_proto,
-	& string_proto,
-	& regex_proto,
-	& array_proto,
-	& refer_proto,
-	& object_proto,
-	NULL
-};
-
+proto_t* bool_proto      = &_bool_proto_;
+proto_t* char_proto      = &_char_proto_;
+proto_t* int_proto       = &_int_proto_;
+proto_t* symbol_proto    = &_symbol_proto_;
+proto_t* float_proto     = &_float_proto_;
+proto_t* file_proto      = &_file_proto_;
+proto_t* dbm_proto       = &_dbm_proto_;
+proto_t* xml_proto       = &_xml_proto_;
+proto_t* function_proto  = &_function_proto_;
+proto_t* class_proto     = &_class_proto_;
+proto_t* label_proto     = &_label_proto_;
+proto_t* frame_proto     = &_frame_proto_;
+proto_t* string_proto    = &_string_proto_;
+proto_t* regex_proto     = &_regex_proto_;
+proto_t* array_proto     = &_array_proto_;
+proto_t* refer_proto     = &_refer_proto_;
+proto_t* object_proto    = &_object_proto_;
